#!/bin/sh

echo "Start time:"
date

if [ $# -ne 1 ]; then
        echo "Usage: aix-audittrail.sh <temp file name>"
        exit -1
fi;

if [ -z "$PROPSFILE" ]; then 
        echo "Value of environment variable PROPSFILE is incorrect."
        exit -1
fi;

TEMPfile=$1
AuditTrailDir=$OPTION2
AuditTrailPrefix=$OPTION3

echo "Parameters passed: $*"

#   Check the options for Audit trail directory", Audit trail prefix if they are not empty use them, else
if [ -z "$AuditTrailDir" -o -z "$AuditTrailPrefix" ]; then
#   Check the blrec database for this values. If they are not empty use them, else return fail result.
        echo "Trying to retrieve parameters from blrec ..."

        DBGETVAL='../bin/dbgetval'

        if [ ! -x $DBGETVAL ]; then
                echo "There is no $DBGETVAL executable."
                exit -1
        fi;

        AuditTrailDir=`$DBGETVAL blrec 'Audit trail directory'` 
        AuditTrailPrefix=`$DBGETVAL blrec 'Audit trail prefix'`
 
        if [ -z "$AuditTrailDir" -o -z "$AuditTrailPrefix" ]; then
                echo "AuditTrailDir or AuditTrailPrefix are empty."
                exit -1
        fi;
fi;

echo "AuditTrailDir    = $AuditTrailDir"
echo "AuditTrailPrefix = $AuditTrailPrefix" 

if [ ! -d $AuditTrailDir ]; then
        echo "Directory $AuditTrailDir doesn't exist."
        exit -2
fi;

#   If /var/log/eprise/working exists wait, because that 
#   means that the audit system is generating a new trail file.
#   Waiting time-out is 5 minutes, when 5 minutes expired and 
#   file still exists script exits with error code.
COUNTER=0
while [ -f /var/log/eprise/working ]; do
        COUNTER=$(($COUNTER+1))
        if [ $COUNTER -gt 300 ]; then
                echo "Exit on timeout! File /var/log/eprise/working still exists"
                exit -3
        fi;
        sleep 1
done

if [ ! -f "$PROPSFILE.trail" ]; then
	find "/$AuditTrailDir" -mtime +2 -name "$AuditTrailPrefix*" -exec rm -f {} \;
	
	CURRENT_DATA=`date`
	echo "First collect at $CURRENT_DATA" > "$PROPSFILE.trail"
fi;

AuditTrailsCounter=`ls /$AuditTrailDir/$AuditTrailPrefix* 2>/dev/null | wc -l`

#   If there are no trail files to be collected create 2 empty files - 
#   non-original sublog file and original sublog file with predefined name 
#   ($PROPSFILE.aix-audittrail.original), then exit with success code (0).
if [ $AuditTrailsCounter -eq 0 ]; then
        echo "There are no audit trail files, so creating empty sublog."

        cat /dev/null > $TEMPfile
        if [ $? -ne 0 ]; then
                echo "Can't create empty $TEMPfile file."
                exit -4
        fi;

        cat /dev/null > $PROPSFILE.aix-audittrail.original
        if [ $? -ne 0 ]; then
                echo "Can't create empty $PROPSFILE.aix-audittrail.original file."
                exit -5
        fi;

        exit 0
fi;

#   Rename all existing trail files to *.collect
#   (i.e. add extra extension .collect to each trail file name)
for loop in `ls /$AuditTrailDir/$AuditTrailPrefix* | grep -v "\.collect$"`
do
        if [ -f "$loop" ]; then
                mv -f "$loop" "$loop.collect"
                if [ $? -ne 0 ]; then
                        echo "Renaming of audit trail file from $loop to $loop.collect has failed."
                        exit -6
                fi;
        else
                echo "File $loop has not been found."
                exit -6
        fi;
done

CollectedTrailsCounter=`ls /$AuditTrailDir/$AuditTrailPrefix*.collect 2>/dev/null | wc -l`

if [ $CollectedTrailsCounter -eq 0 ]; then
        echo "There are no .collect files to collect."
        exit -7
fi;

#   Concatenate all audit trail files ("trail.<date>.collect") together.
/usr/sbin/auditmerge `ls /$AuditTrailDir/$AuditTrailPrefix*.collect | sort` > $PROPSFILE.aix-audittrail.original

if [ $? -ne 0 ]; then
        echo "Merging of .collect files to $PROPSFILE.aix-audittrail.original failed."
        exit -8
fi;

#    Call the collect-trail (see description below) and feed the concatenated file to it.
#    The result will be in TEMPFILE%N% file.
if [ ! -f $PROPSFILE.aix-audittrail.original ]; then
        echo "There is no $PROPSFILE.aix-audittrail.original as demanded."
        exit -9
fi;

../bin/collect-trail -i ./$PROPSFILE.aix-audittrail.original -o $TEMPfile

ERRORCODE=$?
if [ $ERRORCODE -ne 0 ]; then
        echo "collect-trail failed with code $ERRORCODE"
        exit -10
fi;

if [ ! -f $TEMPfile ]; then
        echo "There is no TEMP file [$TEMPfile] as demanded"
        exit -11
fi;

#    Remove all processed .collect files.
rm -f /$AuditTrailDir/$AuditTrailPrefix*.collect

echo "End time:"
date

exit 0

