#!/bin/sh
###########################################################BeginFile############
# Copyright:  (c) 2006 by Consul Risk Management B.V.  All Rights Reserved
#
# Name:
#   getdb2logs.sh
#
# Purpose:
#   Back-end actuator script for IBM DB2 UDB ES for AIX platforms. 
#
# History:
#   060109 0.0.1  ScienceSoft-SA  EE0408009:  Created.
#   060204 0.0.1  ScienceSoft-SA  EE0408009:  Bug fixing.
#   061206        ScienceSoft-SA  EE0609007:  Added DBA mode. 
###########################################################EndFile##############

################################################################################
# do the collection of the records of 2 last elapsed days from file ${delfile} #
################################################################################
first_collect()
{
  DATE_NOW=$(date +"%d-%b-%Y")
  TIME_NOW=$(date +"%H:%M")
  DATE_THEN=$(../bin/goBackInTime $DATE_NOW $TIME_NOW 2880) 

  DD=$(echo $DATE_THEN | sed -e "s/\(.*\)-.*-.* .*:.*/\1/")
  MON=$(echo $DATE_THEN | sed -e "s/.*-\(.*\)-.* .*:.*/\1/")
  MON=`mounth2num $MON`
  YYYY=$(echo $DATE_THEN | sed -e "s/.*-.*-\(.*\) .*:.*/\1/")
  HH=$(echo $DATE_THEN | sed -e "s/.*-.*-.* \(.*\):.*/\1/")
  MM=$(echo $DATE_THEN | sed -e "s/.*-.*-.* .*:\(.*\)/\1/")

  LINE_SHIFT=`wc -l $delfile | sed -e 's/^ *\([0-9]\{1,\}\).*/\1/'` 
  if [ $LINE_SHIFT -eq 0 ] 
  then
    create_empty_file
    return 1
  fi
  
  search_actual
}

####################################################################
# do the actual collection of the new records from file ${delfile} #
####################################################################
work_collect()
{
  line=`wc -l $delfile | sed -e 's/^ *\([0-9]\{1,\}\).*/\1/'` 

  if [ $line -eq 0 ] 
  then
    create_empty_file
    return 1
  fi
  
  first_record=`head -1 $delfile`
  
  if [ $FIRST_RECORD = $first_record ]
  then
    nline=$((LINE_SHIFT+1))
    tail +$nline $delfile > $OUTPUTFILE 2>> $CEA_GENACT_DEBUG_LOG
    [[ $? -ne 0 ]] && our_exit 5
    LINE_SHIFT=$line
    LAST_RECORD=`tail -1 $delfile`
    return 1
  fi

  record_data=$(echo $LAST_RECORD | cut -c2-17)
  YYYY=$(echo $record_data | sed -e "s/\(.*\)-.*-.*-.*\..*/\1/")
  MON=$(echo $record_data | sed -e "s/.*-\(.*\)-.*-.*\..*/\1/")
  DD=$(echo $record_data | sed -e "s/.*-.*-\(.*\)-.*\..*/\1/")
  HH=$(echo $record_data | sed -e "s/.*-.*-.*-\(.*\)\..*/\1/")
  MM=$(echo $record_data | sed -e "s/.*-.*-.*-.*\.\(.*\)/\1/")

  search_actual $LAST_RECORD $record_data
}

##################################################################
# do the collection of the records from file ${delfile} and that #
# are occured not earlier then $YYYY-$MON-$DD $HH:$MM, but after #
# log record $1 (if defined)                                     #
##################################################################
search_actual()
{
  LINE_SHIFT=`wc -l $delfile | sed -e 's/^ *\([0-9]\{1,\}\).*/\1/'` 

  start=1
  start_line=`head -1 $delfile`
  record_data=$(echo $start_line | cut -c2-17)
  if [[ `time_compare $record_data` = "1" ]]
  then
    collect_all_records
    return 1
  fi

  finish=$LINE_SHIFT
  finish_line=`tail -1 $delfile`

  FIRST_RECORD=$start_line
  LAST_RECORD=$finish_line

  record_data=$(echo $finish_line | cut -c2-17)
  if [[ `time_compare $record_data` = "0" ]]
  then
    touch $OUTPUTFILE
    return 1
  fi

  delta=$((finish-start))
  next=$((start+finish))
  next=`expr $next / 2`

  while [ $delta -gt 1 ]
  do
    next_line=`sed -n -e ${next}p $delfile`     
    record_data=$(echo $next_line | cut -c2-17)
    if [[ `time_compare $record_data` = "1" ]]
    then
      finish=$next
    else
      start=$next
    fi
    delta=$((finish-start))
    next=$((start+finish))
    next=`expr $next / 2`
  done

  nline=$((start+1))
  if [ -n "$1" ]
  then
    while [ $nline -le $LINE_SHIFT ]
    do
      next_line=`sed -n -e ${nline}p $delfile`
      record_data=$(echo $next_line | cut -c2-17)
      [[ "$record_data" = "$2" ]] || break;
      if [ "$next_line" = "$1" ]
      then 
        nline=$((nline+1)) 
        break;
      fi
      nline=$((nline+1)) 
    done 
  fi 

  tail +$nline $delfile > $OUTPUTFILE 2>> $CEA_GENACT_DEBUG_LOG
  [[ $? -ne 0 ]] && our_exit 5
}

################################################################
# do the actual collection of all records from file ${delfile} #
################################################################
collect_all_records()
{
  cat $delfile > $OUTPUTFILE 2>> $CEA_GENACT_DEBUG_LOG
  [[ $? -ne 0 ]] && our_exit 5

  LINE_SHIFT=`wc -l $delfile | sed -e 's/^ *\([0-9]\{1,\}\).*/\1/'` 
  if [ ! $LINE_SHIFT -eq 0 ] 
  then
    FIRST_RECORD=`head -1 $delfile`
    LAST_RECORD=`tail -1 $delfile`
  fi
}

####################################################
# perform necessary action using the db2audit tool #
####################################################
db2audit_usage()
{
  debug "The db2audit facility usage started, `date`." >> $CEA_GENACT_DEBUG_LOG
  remove_delfile audit
  remove_delfile checking
  remove_delfile context
  remove_delfile objmaint
  remove_delfile secmaint
  remove_delfile sysadmin
  remove_delfile validate
  if [ -z "$CEA_DB2UDB_DBA_MODE" ]
  then 
    DB2BIN=${INSTHOME}/sqllib/adm/db2audit
    [[ -f $DB2BIN ]] || our_exit 1
  
    su - $OPTION2 -c "DB2INSTANCE=$OPTION2; $DB2BIN flush;" >> $CEA_GENACT_DEBUG_LOG
    [[ $? -ne 0 ]] && our_exit 1

    su - $OPTION2 -c "DB2INSTANCE=$OPTION2; $DB2BIN extract delasc category audit, checking, secmaint, sysadmin, validate;" >> $CEA_GENACT_DEBUG_LOG
    [[ $? -ne 0 ]] && our_exit 1
  else
    . ${INSTHOME}/sqllib/db2profile 2>> $CEA_GENACT_DEBUG_LOG
    [[ $? -ne 0 ]] && our_exit 4 "Error while ${INSTHOME}/sqllib/db2profile script execution."
  
    db2audit flush >> $CEA_GENACT_DEBUG_LOG
    [[ $? -ne 0 ]] && our_exit 1

    db2audit extract delasc category audit, checking, secmaint, sysadmin, validate >> $CEA_GENACT_DEBUG_LOG
    [[ $? -ne 0 ]] && our_exit 1    
  fi
  debug "The db2audit facility usage finished, `date`." >> $CEA_GENACT_DEBUG_LOG
}

create_empty_file()
{
  touch $OUTPUTFILE
  FIRST_RECORD=""
  LAST_RECORD=""
  LINE_SHIFT=0
}

our_exit()
{
  case "$1"
  in
    1) debug "Error while the audit facility administrator tool ran (return code 1)"
       finish_log
       exit 1;; 
    3) debug "Some of the pre-existing desired log files ($2.del) could not" \
       "be removed before extracting the new ones (return code 3)"
       finish_log
       exit 3;;
    4) debug $2 
       finish_log
       exit 4;;
  esac   

  [[ $1 -eq 0 ]] && AUDITERROR=0

  if [ $RUNCOUNTER -eq 5 ] 
  then
    case "$AUDITERROR"
    in
      0) debug "The collect process was a success (return code 0)"
         finish_log
         exit 0;; 
      1) debug "None of the desired log files could be accessed (return code 2)"
         finish_log
         exit 2;; 
    esac   
  fi

  echo "RUNCOUNTER=$RUNCOUNTER" > $TEMPFILE
  echo "AUDITERROR=$AUDITERROR" > $TEMPFILE2
  exit 0
}

finish_log()
{
  rm -f $TEMPFILE
  rm -f $TEMPFILE2
  echo "CeA/ExtendIT Debug Log finished by $SCRIPTNAME on `hostname` (`uname`), `date`." >> $CEA_GENACT_DEBUG_LOG
}

remove_delfile()
{
  rmfile=${DB2LOGPATH}/"$1".del 
  if [ -f $rmfile ]
  then
    if [ -z "$CEA_DB2UDB_DBA_MODE" ]
    then 
      su $OPTION2 -c "rm $rmfile" 2>> $CEA_GENACT_DEBUG_LOG
      [ $? -ne 0 ] && our_exit 3 $rmfile
    else
      rm $rmfile 2>> $CEA_GENACT_DEBUG_LOG
      [ $? -ne 0 ] && our_exit 3 $rmfile
    fi
  fi
}

get_instance_home()
{
  if [ -z "$CEA_DB2UDB_DBA_MODE" ]
  then 
    db2home=`su - $OPTION2 -c 'echo $INSTHOME'|tail -1`
    [ $? -ne 0 ] && our_exit 2
  
    if [ -z "$db2home" ] 
    then
      db2home=`su - $OPTION2 -c 'echo $HOME'|tail -1`
      [ $? -ne 0 ] && our_exit 2
    fi

    if [ -z "$db2home" ] 
    then
      db2home=/home/${OPTION2}
    fi
  else
    db2home=`cat /etc/passwd | grep "^${OPTION2}:" | sed -e "s/^${OPTION2}:[^:]*:[^:]*:[^:]*:[^:]*:\([^:]*\):[^:]*/\1/"`
    [ $? -ne 0 ] && our_exit 4 "Error while reading /etc/passwd in order to find audited instance home directory."
  fi
  echo $db2home
}

mounth2num () {  
  case $1 in
    'Jan')
       outval="01"
       ;;
    'Feb')
       outval="02"
       ;;
    'Mar')
       outval="03"
       ;;
    'Apr')
       outval="04"
       ;;
    'May')
       outval="05"
       ;;
    'Jun')
       outval="06"
       ;;
    'Jul')
       outval="07"
       ;;
    'Aug')
       outval="08"
       ;;
    'Sep')
       outval="09"
       ;;
    'Oct')
       outval="10"
       ;;
    'Nov')
       outval="11"
       ;;
    'Dec')
       outval="12"
       ;;
     *)
       outval="00"
       ;;
  esac
  echo $outval
}

compv() {
  if [ $1 -gt $2 ]
  then
    echo "0"
  else
    echo "1"
  fi
}

time_compare()
{
  YYYY2=$(echo $1 | sed -e "s/\(.*\)-.*-.*-.*\..*/\1/")
  if [ $YYYY -eq $YYYY2 ]
  then
    MON2=$(echo $1 | sed -e "s/.*-\(.*\)-.*-.*\..*/\1/")
    if [ $MON -eq $MON2 ]
    then
      DD2=$(echo $1 | sed -e "s/.*-.*-\(.*\)-.*\..*/\1/")
      if [ $DD -eq $DD2 ]
      then
        HH2=$(echo $1 | sed -e "s/.*-.*-.*-\(.*\)\..*/\1/")
        if [ $HH -eq $HH2 ]
        then
          MM2=$(echo $1 | sed -e "s/.*-.*-.*-.*\.\(.*\)/\1/")
          echo `compv $MM $MM2`
        else
          echo `compv $HH $HH2`
        fi
      else
        echo `compv $DD $DD2`
      fi
    else
      echo `compv $MON $MON2`
    fi
  else
    echo `compv $YYYY $YYYY2`
  fi
}

store_state()
{
  echo ${LOGFILENAME}.del_FIRSTRECORD=$FIRST_RECORD >> $PROPSFILE
  echo ${LOGFILENAME}.del_LASTRECORD=$LAST_RECORD >> $PROPSFILE
  echo ${LOGFILENAME}.del_LINESHIFT=$LINE_SHIFT >> $PROPSFILE
}

CEA_GENACT_DEBUG_LOG=../log/${PROPSFILE}.log
. $CEA_GENACT_FUNCLIB || exit 1

# Validate input 

[[ $# -ge 1 ]] \
  || our_exit 4 "Insufficient number of arguments."

[[ -n "$OPTION2" ]] \
  || our_exit 4 "Option 'Instance name' must be specified"

[[ -n "$PROPSFILE" ]] \
  || our_exit 4 "Properties file must be specified"

# We expect the name of the outfile in $1 
OUTPUTFILE=$1

. ${OPTION1} || our_exit 4 "Can not read ${OPTION1}." 

TEMPFILE=${PROPSFILE}.tmp
TEMPFILE2=${PROPSFILE}.tmp2

if [ -r "$TEMPFILE" ]
then
  . ./$TEMPFILE || our_exit 4 "Internal error"
  if [[ $RUNCOUNTER -eq 5 ]]
  then
    RUNCOUNTER=1
    AUDITERROR=1
  else
    RUNCOUNTER=$((RUNCOUNTER+1))
    . ./$TEMPFILE2 || our_exit 4 "Internal error"
  fi
else
  RUNCOUNTER=1
  AUDITERROR=1
fi

INSTHOME=`get_instance_home`
DB2LOGPATH=${INSTHOME}/sqllib/security

case "$RUNCOUNTER"
in
    1) db2audit_usage
       LOGFILENAME=audit;;
    2) LOGFILENAME=checking;;
    3) LOGFILENAME=secmaint;;
    4) LOGFILENAME=sysadmin;;
    5) LOGFILENAME=validate;;
esac

delfile=${DB2LOGPATH}/${LOGFILENAME}.del
[[ -r "$delfile" ]] || our_exit 2

if [ ! -r "$PROPSFILE" ]
then
  first_collect 
else
  LINE_SHIFT=`cat $PROPSFILE | grep "^${LOGFILENAME}.del_LINESHIFT=" | sed -e "s/^${LOGFILENAME}.del_LINESHIFT=\(.*\)/\1/"`
  if [ -z "$LINE_SHIFT" ]
  then
    first_collect
  else
    if [ $LINE_SHIFT -eq 0 ]
    then
      collect_all_records
    else
      FIRST_RECORD=`cat $PROPSFILE | grep "^${LOGFILENAME}.del_FIRSTRECORD=" | sed -e "s/^${LOGFILENAME}.del_FIRSTRECORD=\(.*\)/\1/"`
      if [ -z "$FIRST_RECORD" ]
      then
        first_collect
      else
        LAST_RECORD=`cat $PROPSFILE | grep "^${LOGFILENAME}.del_LASTRECORD=" | sed -e "s/^${LOGFILENAME}.del_LASTRECORD=\(.*\)/\1/"`
        if [ -z "$LAST_RECORD" ]
        then
          first_collect
        else
          work_collect
        fi
      fi   
    fi
  fi
fi

if [ ! -r "$PROPSFILE" ]
then
  echo "encoding1=UTF-8" >> $PROPSFILE 
  echo "encoding2=UTF-8" >> $PROPSFILE 
  echo "encoding3=UTF-8" >> $PROPSFILE 
  echo "encoding4=UTF-8" >> $PROPSFILE 
  echo "encoding5=UTF-8" >> $PROPSFILE
  echo "_logpath=${DB2LOGPATH}" >> $PROPSFILE 
  store_state
else
  cat $PROPSFILE | grep -v "^${LOGFILENAME}.del_" > ${PROPSFILE}1 ; 
  cat ${PROPSFILE}1 > $PROPSFILE 
  store_state 
  rm ${PROPSFILE}1
fi

our_exit 0
