#!/bin/ksh
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

gobackintime()
{
CURRENT_YEAR=`date +%Y`
CURRENT_MONTH=`date +%m`
CURRENT_DAY=`date +%d`

# checking for leap year
LEAP=0

if [ `expr $CURRENT_YEAR % 400` -eq 0 ] ; then
 LEAP=1
else
 if [ `expr $CURRENT_YEAR % 100` -eq 0 ] ; then 
   LEAP=0
 else
  if [ `expr $CURRENT_YEAR % 4` -eq 0 ] ; then
   LEAP=1
  fi  
 fi 
fi

NEW_DAY=`expr $CURRENT_DAY - 2`
NEW_MONTH=$CURRENT_MONTH
NEW_YEAR=$CURRENT_YEAR

if [ $NEW_DAY -lt 1 ] ; then
case $CURRENT_MONTH in
 1)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=12
   NEW_YEAR=`expr $CURRENT_YEAR - 1`
   ;;
 2)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 3)
   if [ $LEAP -eq 1 ] ; then
    NEW_DAY=`expr $NEW_DAY + 29`
    NEW_MONTH=`expr $CURRENT_MONTH - 1`
   else
    NEW_DAY=`expr $NEW_DAY + 28`
    NEW_MONTH=`expr $CURRENT_MONTH - 1`
   fi
   ;;
 4)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 5)
   NEW_DAY=`expr $NEW_DAY + 30`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 6)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 7)
   NEW_DAY=`expr $NEW_DAY + 30`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 8)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 9)
   NEW_DAY=`expr $NEW_DAY + 30`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 10)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 11)
   NEW_DAY=`expr $NEW_DAY + 30`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 12)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 *) echo Error checking month number. >> $ACTUATOR_LOG
   ;;
esac
fi

# adding "0" character if number contains only one digit
NEW_MONTH=`echo $NEW_MONTH | sed "s/^[1-9]$/0\$NEW_MONTH/"`
NEW_DAY=`echo $NEW_DAY | sed "s/^[1-9]$/0\$NEW_DAY/"`
echo "$NEW_YEAR-$NEW_MONTH-$NEW_DAY"
}

#ACTUATOR_LOG=../log/actuator.log
ACTUATOR_LOG=../log/$PROPSFILE.log

# checking instance name

if [ "a$TEMPFILE1" = "a" ] ; then 
 echo `date "+%a %D %T"` 1-st sublog name is not defined. Exiting. >> $ACTUATOR_LOG 
 exit 1
fi

if [ "a$OPTION2" != "a" ] ; then 
 INSTANCE_ARCHIVE_PATH=$OPTION2
else
 echo `date "+%a %D %T"` Instance level archive path is not defined. Exiting. >> $ACTUATOR_LOG 
 exit 1
fi

if [ "a$OPTION3" != "a" ] ; then 
 INSTANCE_FILE_PATTERN=$OPTION3
else
 echo `date "+%a %D %T"` Instance level pattern is not defined. Exiting. >> $ACTUATOR_LOG 
 exit 1
fi

if [ "a$PROPSFILE" = "a" ] ; then 
 echo `date "+%a %D %T"` PROPSFILE parameter is not defined. Exiting. >> $ACTUATOR_LOG 

 exit 1
fi

if [ "a$OPTION6" != "a" ] ; then
 INSTANCE_NAME=$OPTION6
else
 echo `date "+%a %D %T"` Instance name is not defined. Exiting. >> $ACTUATOR_LOG
 exit 1
fi

# checking who runs the script
RUN_AS_ROOT=0
ID=`id | sed -e 's/uid=//; s/(.*//'`
if [ $ID -eq 0 ] ; then
  RUN_AS_ROOT=1
else
  DB2AUDIT_PATH=`cat /etc/passwd | grep "^$INSTANCE_NAME:" | sed -e "s/^$INSTANCE_NAME:[^:]*:[^:]*:[^:]*:[^:]*:\([^:]*\):[^:]*/\1/"`/sqllib/adm
  if [ ! $? -eq 0 ] ; then
   echo `date "+%a %D %T"` Error reading /etc/passwd file. Exiting. >> $ACTUATOR_LOG
   return 1
  fi
  DB2INSTANCE=$INSTANCE_NAME
  export DB2INSTANCE
fi

# internal variables
INST_DISABLE=0
if [ $RUN_AS_ROOT -eq 1 ] ; then
 CURRENT_DIR=`su - $INSTANCE_NAME -c "pwd" | tail -1`
else
 CURRENT_DIR=`pwd`
fi
TEMP_DIR="$CURRENT_DIR/$PROPSFILE.dir"
ARCHLIST_TMP=$TEMP_DIR/archlist_tmp.txt
ARCHLIST=$TEMP_DIR/archlist.txt
ALL_INST_ARCHLIST=$TEMP_DIR/all_inst_archlist.txt
# only FOR TEST!!!!!!!!
DATE_BEFORE=`gobackintime | sed -e 's/-//g'``date +%H%M%S`
#DATE_BEFORE=20070823235847
TMPFILE=$TEMP_DIR/file.txt
NEW_PROPS_FILE=$TEMP_DIR/props_file.new
TEMP_PROPS_FILE=$TEMP_DIR/props_file.tmp

rm -rf $TEMP_DIR

#creating temporary directory
if [ $RUN_AS_ROOT -eq 1 ] ; then
 if [ ! -d $TEMP_DIR ] ; then 
  su - $INSTANCE_NAME -c "mkdir $TEMP_DIR "
 fi
else
 if [ ! -d $TEMP_DIR ] ; then 
  mkdir $TEMP_DIR
 fi
fi


# cleaning output files
if [ -f $TEMPFILE1 ] ; then
 rm -f $TEMPFILE1 
fi
if [ -f $ALL_INST_ARCHLIST ] ; then
 rm -f $ALL_INST_ARCHLIST 
fi
#cleaning temporary files
if [ -f $TEMP_PROPS_FILE ] ; then
 rm -f $TEMP_PROPS_FILE
fi
if [ -f $NEW_PROPS_FILE ] ; then
 rm -f $NEW_PROPS_FILE
fi

touch $TEMPFILE1
chmod 666 $TEMPFILE1

# checking whether database level archive path exists
if [ "$INSTANCE_ARCHIVE_PATH" = "skip" ] ; then
 # disabling instance
 echo `date "+%a %D %T"` Instance level audit is disabled. >> $ACTUATOR_LOG
 touch $TEMPFILE1
 return 0
fi
if [ "$INSTANCE_ARCHIVE_PATH" = "default" ] ; then
 if [ $RUN_AS_ROOT -eq 1 ] ; then
  INSTANCE_ARCHIVE_PATH=`su - $INSTANCE_NAME -c 'db2audit describe | grep Archive | cut -d \" -f 2' | tail -1`
  #su - $INSTANCE_NAME -c 'db2audit describe | grep Archive | cut -d \" -f 2 ' 
 else
  INSTANCE_ARCHIVE_PATH=`$DB2AUDIT_PATH/db2audit describe | grep Archive | cut -d \" -f 2`
 fi
fi
if [ ! -d "$INSTANCE_ARCHIVE_PATH" ] ; then
 # disabling instance
 INST_DISABLE=1 
 echo `date "+%a %D %T"` Instances archive path not valid - Instance level audit is disabled. >> $ACTUATOR_LOG
 touch $TEMPFILE1
 return 0 
fi

INSTANCE_ARCHIVE_PATH=`echo $INSTANCE_ARCHIVE_PATH | sed -e 's/ /\\\ /g'`

# performing collect
echo `date "+%a %D %T"` Processing of instance level text dumps started.  > $ACTUATOR_LOG

if [ -f $ARCHLIST_TMP ] ; then 
  rm -f $ARCHLIST_TMP 
fi

if [ $RUN_AS_ROOT -eq 1 ] ; then
 su - $INSTANCE_NAME -c "db2audit archive to $INSTANCE_ARCHIVE_PATH/"
 if [ ! $? -eq 0 ] ; then
  echo `date "+%a %D %T"` db2audit archive run error. >> $ACTUATOR_LOG
 fi
else

#$DB2AUDIT_PATH/db2audit archive to "$INSTANCE_ARCHIVE_PATH/"
/bin/ksh -c "$DB2AUDIT_PATH/db2audit archive to $INSTANCE_ARCHIVE_PATH/"
 if [ ! $? -eq 0 ] ; then
  echo `date "+%a %D %T"` db2audit archive run error. >> $ACTUATOR_LOG 
 fi
fi
PATTERN=`echo $INSTANCE_FILE_PATTERN | sed -e "s/\*/$i/" `

/bin/ksh -c "ls -t -r $INSTANCE_ARCHIVE_PATH | grep ^$PATTERN.* | grep \\.[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]$" >> $ARCHLIST_TMP

if [ -f $ARCHLIST ] ; then
 rm -f $ARCHLIST
fi
#reading last timestamp from propsfile
if [ -f $PROPSFILE ] ; then
 DATE_BEFORE=`cat $PROPSFILE | grep $PATTERN | sed -e 's/^.*\.log\..\.//g'`
fi

for k in `cat $ARCHLIST_TMP`; do
 CURRENT_ARCHIVE_TIMESTAMP=`echo $k | sed -e 's/^.*\.log\..\.//g'` 

 COMPARE=`expr "$CURRENT_ARCHIVE_TIMESTAMP" \> "$DATE_BEFORE"`
 if [ $COMPARE -eq 1 ] ; then
  echo $k >> $ARCHLIST  
 fi
done

# checking if there is data to collect
if [ ! -f $ARCHLIST ] ; then
 echo `date "+%a %D %T"` There are no data to collect. >> $ACTUATOR_LOG
 echo `date "+%a %D %T"` Possible reasons: instance level path or pattern are wrong or there are no new events. Exiting. >> $ACTUATOR_LOG
 touch $TEMPFILE1
 return 0
fi

#extracting archives into TEMP_FILE1
cat $ARCHLIST >> $ALL_INST_ARCHLIST 

for k in `cat $ARCHLIST`; do
if [ $RUN_AS_ROOT -eq 1 ] ; then
 echo `date "+%a %D %T"` Extracting archive $k  >> $ACTUATOR_LOG
 su - $INSTANCE_NAME -c "db2audit extract file $TEMPFILE1 from files $INSTANCE_ARCHIVE_PATH/$k" | grep Operation >> $ACTUATOR_LOG 
 if [ ! $? -eq 0 ] ; then
  echo `date "+%a %D %T"` db2audit unable to extract archive $k. >> $ACTUATOR_LOG
 fi
else
 echo `date "+%a %D %T"` Extracting archive $k  >> $ACTUATOR_LOG
  #$DB2AUDIT_PATH/db2audit extract file $TEMPFILE1 from files "$INSTANCE_ARCHIVE_PATH/$k" | grep Operation >> $ACTUATOR_LOG 
  /bin/ksh -c "$DB2AUDIT_PATH/db2audit extract file $TEMPFILE1 from files $INSTANCE_ARCHIVE_PATH/$k | grep Operation" >> $ACTUATOR_LOG
  if [ ! $? -eq 0 ] ; then
   echo `date "+%a %D %T"` db2audit unable to extract archive $k. >> $ACTUATOR_LOG
  fi
 fi
 PROPS_FILE_LINE="lastinstfile=$k"
done
 #creating temporary propsfile
 if [ "a$PROPS_FILE_LINE" != "a" ] ; then
  echo $PROPS_FILE_LINE >> $TEMP_PROPS_FILE
 fi
 PROPS_FILE_LINE=""

 # merging propsfiles
 for i in `cat $PROPSFILE`; do
  PROPSSTRING=""
  TEMP=""
  TEMP=`echo $i | sed -e 's/\.[0-9]*$//g'`
  PROPSSTRING=`cat $TEMP_PROPS_FILE | grep $TEMP`
  if [ "a$PROPSSTRING" = "a" ] ; then
   echo $i >> $NEW_PROPS_FILE
  fi
 done
 if [ -f $NEW_PROPS_FILE ] ; then
  cp $NEW_PROPS_FILE  $PROPSFILE
  cat $TEMP_PROPS_FILE >> $PROPSFILE
 else
  cp $TEMP_PROPS_FILE  $PROPSFILE
 fi

#checking if there some data was extracted
if [ ! -f $TEMPFILE1 ] ; then
 touch $TEMPFILE1
fi

#clearing temporary directory
if [ -f $ARCHLIST ] ; then
 rm -f $ARCHLIST
fi
if [ -f $ARCHLIST_TMP ] ; then
 rm -f $ARCHLIST_TMP
fi
if [ -f $TEMP_PROPS_FILE ] ; then
 rm -f $TEMP_PROPS_FILE
fi
if [ -f $NEW_PROPS_FILE ] ; then
 rm -f $NEW_PROPS_FILE
fi

echo `date "+%a %D %T"` Processing of instance level text dumps finished.>> $ACTUATOR_LOG
echo `date "+%a %D %T"` Collect of instance level text dumps successful. >> $ACTUATOR_LOG
return 0
