#!/bin/ksh
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

gobackintime()
{
CURRENT_YEAR=`date +%Y`
CURRENT_MONTH=`date +%m`
CURRENT_DAY=`date +%d`

# checking for leap year
LEAP=0

if [ `expr $CURRENT_YEAR % 400` -eq 0 ] ; then
 LEAP=1
else
 if [ `expr $CURRENT_YEAR % 100` -eq 0 ] ; then 
   LEAP=0
 else
  if [ `expr $CURRENT_YEAR % 4` -eq 0 ] ; then
   LEAP=1
  fi  
 fi 
fi

NEW_DAY=`expr $CURRENT_DAY - 2`
NEW_MONTH=$CURRENT_MONTH
NEW_YEAR=$CURRENT_YEAR

if [ $NEW_DAY -lt 1 ] ; then
case $CURRENT_MONTH in
 1)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=12
   NEW_YEAR=`expr $CURRENT_YEAR - 1`
   ;;
 2)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 3)
   if [ $LEAP -eq 1 ] ; then
    NEW_DAY=`expr $NEW_DAY + 29`
    NEW_MONTH=`expr $CURRENT_MONTH - 1`
   else
    NEW_DAY=`expr $NEW_DAY + 28`
    NEW_MONTH=`expr $CURRENT_MONTH - 1`
   fi
   ;;
 4)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 5)
   NEW_DAY=`expr $NEW_DAY + 30`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 6)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 7)
   NEW_DAY=`expr $NEW_DAY + 30`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 8)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 9)
   NEW_DAY=`expr $NEW_DAY + 30`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 10)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 11)
   NEW_DAY=`expr $NEW_DAY + 30`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 12)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 *) echo Error checking month number. >> $ACTUATOR_LOG
   ;;
esac
fi

# adding "0" character if number contains only one digit
NEW_MONTH=`echo $NEW_MONTH | sed "s/^[1-9]$/0\$NEW_MONTH/"`
NEW_DAY=`echo $NEW_DAY | sed "s/^[1-9]$/0\$NEW_DAY/"`
echo "$NEW_YEAR-$NEW_MONTH-$NEW_DAY"
}

ACTUATOR_LOG=../log/$PROPSFILE.log

if [ "a$TEMPFILE2" = "a" ] ; then 
 echo `date "+%a %D %T"` 2-nd sublog name is not defined. Exiting. >> $ACTUATOR_LOG 
 exit 1
fi

if [ "a$OPTION4" != "a" ] ; then 
 DATABASE_ARCHIVE_PATH=$OPTION4
else
 echo `date "+%a %D %T"` Database level archive path is not defined. Exiting. >> $ACTUATOR_LOG 
 exit 1
fi

if [ "a$OPTION5" != "a" ] ; then 
 DATABASE_FILE_PATTERN=$OPTION5
else
 echo `date "+%a %D %T"` Database level pattern is not defined. Exiting. >> $ACTUATOR_LOG 
 exit 1
fi

if [ "a$OPTION6" != "a" ] ; then 
 INSTANCE_NAME=$OPTION6
else
 echo `date "+%a %D %T"` Instance name is not defined. Exiting. >> $ACTUATOR_LOG 
 exit 1
fi

if [ "a$PROPSFILE" = "a" ] ; then 
 echo `date "+%a %D %T"` PROPSFILE is not defined. Exiting. >> $ACTUATOR_LOG 
 exit 1
fi

# checking who runs the script
RUN_AS_ROOT=0
ID=`id | sed -e 's/uid=//; s/(.*//'`
if [ $ID -eq 0 ] ; then
  RUN_AS_ROOT=1
else
  DB2AUDIT_PATH=`cat /etc/passwd | grep "^$INSTANCE_NAME:" | sed -e "s/^$INSTANCE_NAME:[^:]*:[^:]*:[^:]*:[^:]*:\([^:]*\):[^:]*/\1/"`/sqllib/adm
  if [ ! $? -eq 0 ] ; then
   echo `date "+%a %D %T"` Error reading /etc/passwd file. Exiting. >> $ACTUATOR_LOG
   return 1
  fi
  DB2_PATH=`cat /etc/passwd | grep "^$INSTANCE_NAME:" | sed -e "s/^$INSTANCE_NAME:[^:]*:[^:]*:[^:]*:[^:]*:\([^:]*\):[^:]*/\1/"`/sqllib/bin
  if [ ! $? -eq 0 ] ; then
   echo `date "+%a %D %T"` Error reading /etc/passwd file. Exiting. >> $ACTUATOR_LOG
   return 1
  fi
  DB2INSTANCE=$INSTANCE_NAME
  export DB2INSTANCE
fi


# internal variables
if [ $RUN_AS_ROOT -eq 1 ] ; then
 CURRENT_DIR=`su - $INSTANCE_NAME -c "pwd" | tail -1`
else
 CURRENT_DIR=`pwd`
fi
TEMP_DIR="$CURRENT_DIR/$PROPSFILE.dir"
INST_DISABLE=0
DB_DISABLE=0
DATABASE_LIST_TMP=$TEMP_DIR/database_list_tmp.txt
DATABASE_LIST=$TEMP_DIR/database_list.txt
ARCHLIST_TMP=$TEMP_DIR/archlist_tmp.txt
ARCHLIST=$TEMP_DIR/archlist.txt
ALL_DB_ARCHLIST=$TEMP_DIR/all_db_archlist.txt
# only FOR TEST!!!!!!!!
DATE_BEFORE=`gobackintime | sed -e 's/-//g'``date +%H%M%S`
TMPFILE=$TEMP_DIR/file.txt
NEW_PROPS_FILE=$TEMP_DIR/props_file.new
TEMP_PROPS_FILE=$TEMP_DIR/props_file.tmp


#checking input parameters
echo `date "+%a %D %T"` Processing of database level text dumps started.  >> $ACTUATOR_LOG


# checking for temporary directory
if [ ! -d $TEMP_DIR ] ; then 
 echo `date "+%a %D %T"` Temporary directory doesn\'t exist. Exiting.  >> $ACTUATOR_LOG
 return 1 
fi

# cleaning output files
if [ -f $TEMPFILE2 ] ; then
 rm -f $TEMPFILE2 
fi
if [ -f $ALL_DB_ARCHLIST ] ; then
 rm -f $ALL_DB_ARCHLIST 
fi
#cleaning temporary files
if [ -f $TEMP_PROPS_FILE ] ; then
 rm -f $TEMP_PROPS_FILE
fi
if [ -f $NEW_PROPS_FILE ] ; then
 rm -f $NEW_PROPS_FILE
fi



# checking whether database level archive path exists
if [ "$DATABASE_ARCHIVE_PATH" = "skip" ] ; then
 # disabling instance
 echo `date "+%a %D %T"` Database level audit is disabled. >> $ACTUATOR_LOG
 touch $TEMPFILE2
 return 0
fi
if [ "$DATABASE_ARCHIVE_PATH" = "default" ] ; then
 if [ $RUN_AS_ROOT -eq 1 ] ; then
  DATABASE_ARCHIVE_PATH=`su - $INSTANCE_NAME -c 'db2audit describe | grep Archive | cut -d \" -f 2' | tail -1`
 else
  DATABASE_ARCHIVE_PATH=`$DB2AUDIT_PATH/db2audit describe | grep Archive | cut -d \" -f 2`
 fi
fi
if [ ! -d "$DATABASE_ARCHIVE_PATH" ] ; then
 # disabling instance
 DB_DISABLE=1 
 echo `date "+%a %D %T"` Database level audit is disabled. >> $ACTUATOR_LOG
 touch $TEMPFILE2
 return 0 
fi

DATABASE_ARCHIVE_PATH=`echo $DATABASE_ARCHIVE_PATH | sed -e 's/ /\\\ /g'`

touch $TEMPFILE2
chmod 666 $TEMPFILE2

# performing collect

# getting list of the databases
if [ RUN_AS_ROOT -eq 0 ] ; then
 $DB2_PATH/db2 list database directory > $DATABASE_LIST_TMP
 if [ ! $? -eq 0 ] ; then
  echo `date "+%a %D %T"` db2 run error. >> $ACTUATOR_LOG
  echo `date "+%a %D %T"` Collect of database level text dumps failed. Exiting. >> $ACTUATOR_LOG
  touch $TEMPFILE2
  return 0 
 fi
else
 su - "$INSTANCE_NAME" -c "db2 list database directory" > $DATABASE_LIST_TMP
 if [ ! $? -eq 0 ] ; then
  echo `date "+%a %D %T"` db2 run error. >> $ACTUATOR_LOG
  echo `date "+%a %D %T"` Collect of database level text dumps failed. Exiting. >> $ACTUATOR_LOG
  touch $TEMPFILE2
  return 0 
 fi
fi
#getting list of databases
cat $DATABASE_LIST_TMP | grep "Database name"  | cut -d = -f 2 | sed -e 's/^ //'> $DATABASE_LIST

for i in `cat $DATABASE_LIST`; do
 if [ -f $ARCHLIST_TMP ] ; then 
   rm -f $ARCHLIST_TMP 
 fi
 echo working for database $i:
 if [ $RUN_AS_ROOT -eq 1 ] ; then
  su - $INSTANCE_NAME -c "db2audit archive database $i to $DATABASE_ARCHIVE_PATH/"
  if [ ! $? -eq 0 ] ; then
   echo `date "+%a %D %T"` db2audit archive run error. >> $ACTUATOR_LOG
  fi
 else
  #$DB2AUDIT_PATH/db2audit archive database $i to "$DATABASE_ARCHIVE_PATH/"
  /bin/ksh -c "$DB2AUDIT_PATH/db2audit archive database $i to $DATABASE_ARCHIVE_PATH/"
  if [ ! $? -eq 0 ] ; then
   echo `date "+%a %D %T"` db2audit archive run error. >> $ACTUATOR_LOG
  fi
 fi
 PATTERN=`echo $DATABASE_FILE_PATTERN | sed -e "s/\*/$i/" `

 /bin/ksh -c "ls -t -r $DATABASE_ARCHIVE_PATH | grep ^$PATTERN.* | grep \\.[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]$" >> $ARCHLIST_TMP

 if [ -f $ARCHLIST ] ; then
  rm -f $ARCHLIST
 fi
 #reading last timestamp from propsfile
 DATE_BEFORE=`cat $PROPSFILE | grep $PATTERN | sed -e 's/^.*\.log\..\.//g'`
 # if there are no record for current database in propsfile
 if [ "a$DATE_BEFORE" = "a" ] ; then
  DATE_BEFORE=`gobackintime | sed -e 's/-//g'``date +%H%M%S`
 fi

 for k in `cat $ARCHLIST_TMP`; do
  CURRENT_ARCHIVE_TIMESTAMP=`echo $k | sed -e 's/^.*\.log\..\.//g'` 
  COMPARE=`expr "$CURRENT_ARCHIVE_TIMESTAMP" \> "$DATE_BEFORE"`
  if [ $COMPARE -eq 1 ] ; then
   echo $k >> $ARCHLIST  
 fi
 done

 #checking if there are data to collect
 if [ ! -f $ARCHLIST ] ; then
  echo `date "+%a %D %T"` There are no data to collect for database $i. >> $ACTUATOR_LOG
  echo `date "+%a %D %T"` Possible reasons: database level path, instance name or pattern are wrong or there are no new events. >> $ACTUATOR_LOG
 fi

 #extracting archives into TEMPFILE2
 cat $ARCHLIST >> $ALL_DB_ARCHLIST 
 for k in `cat $ARCHLIST`; do
  if [ $RUN_AS_ROOT -eq 1 ] ; then
   echo `date "+%a %D %T"` Extracting archive $k >> $ACTUATOR_LOG
   su - $INSTANCE_NAME -c "db2audit extract file $TEMPFILE2 from files $DATABASE_ARCHIVE_PATH/$k" | grep Operation >> $ACTUATOR_LOG 
   if [ ! $? -eq 0 ] ; then
    echo `date "+%a %D %T"` db2audit unable to extract archive $k. >> $ACTUATOR_LOG
   fi
  else
   echo `date "+%a %D %T"` Extracting archive $k >> $ACTUATOR_LOG
   #$DB2AUDIT_PATH/db2audit extract file $TEMPFILE2 from files "$DATABASE_ARCHIVE_PATH/$k" | grep Operation >> $ACTUATOR_LOG 
   /bin/ksh -c "$DB2AUDIT_PATH/db2audit extract file $TEMPFILE2 from files $DATABASE_ARCHIVE_PATH/$k | grep Operation" >> $ACTUATOR_LOG 
   if [ ! $? -eq 0 ] ; then
    echo `date "+%a %D %T"` db2audit unable to extract archive $k. >> $ACTUATOR_LOG
   fi
  fi
  PROPS_FILE_LINE="lastdbfile=$k"
 done
  #creating temporary propsfile
  if [ "a$PROPS_FILE_LINE" != "a" ] ; then
   echo $PROPS_FILE_LINE >> $TEMP_PROPS_FILE
  fi
  PROPS_FILE_LINE=""

done 
 # merging propsfiles
 for i in `cat $PROPSFILE`; do
  PROPSSTRING=""
  TEMP=""
  TEMP=`echo $i | sed -e 's/\.[0-9]*$//g'`
  PROPSSTRING=`cat $TEMP_PROPS_FILE | grep $TEMP`
  if [ "a$PROPSSTRING" = "a" ] ; then
   echo $i >> $NEW_PROPS_FILE
  fi
 done
 if [ -f $NEW_PROPS_FILE ] ; then 
  cp $NEW_PROPS_FILE  $PROPSFILE
  cat $TEMP_PROPS_FILE >> $PROPSFILE
 else 
  cp $TEMP_PROPS_FILE  $PROPSFILE
 fi

#checking if there some data was extracted
if [ ! -f $TEMPFILE2 ] ; then
 touch $TEMPFILE2
fi
 
#clearing temporary directory
if [ -f $ARCHLIST ] ; then
 rm -f $ARCHLIST
fi
if [ -f $ARCHLIST_TMP ] ; then
 rm -f $ARCHLIST_TMP
fi
if [ -f $DATABASE_LIST_TMP ] ; then
 rm -f $DATABASE_LIST_TMP 
fi
if [ -f $DATABASE_LIST ] ; then
 rm -f $DATABASE_LIST 
fi
if [ -f $TEMP_PROPS_FILE ] ; then
 rm -f $TEMP_PROPS_FILE
fi
if [ -f $NEW_PROPS_FILE ] ; then
 rm -f $NEW_PROPS_FILE
fi

echo `date "+%a %D %T"` Collect of database level text dumps successful. >> $ACTUATOR_LOG
return 0
