#!/bin/ksh
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

ACTUATOR_LOG=../log/$PROPSFILE.log
#checking input parameters
echo `date "+%a %D %T"` Processing of database level original logs is started.  >> $ACTUATOR_LOG

if [ "a$TEMPFILE4" = "a" ] ; then 
 echo `date "+%a %D %T"`  4-th sublog name is not defined. Exiting. >> $ACTUATOR_LOG 
 exit 1
fi

if [ "a$OPTION4" != "a" ] ; then 
 DATABASE_ARCHIVE_PATH=$OPTION4
else
 echo `date "+%a %D %T"` Database level archive path is not defined. Exiting. >> $ACTUATOR_LOG 
 exit 1
fi

if [ "a$OPTION6" != "a" ] ; then 
 INSTANCE_NAME=$OPTION6
else
 echo `date "+%a %D %T"` Instance name is not defined. Exiting. >> $ACTUATOR_LOG 
 exit 1
fi

if [ "a$PROPSFILE" = "a" ] ; then 
 echo `date "+%a %D %T"` PROPSFILE is not defined. Exiting. >> $ACTUATOR_LOG 
 exit 1
fi

# checking who runs the script
RUN_AS_ROOT=0
ID=`id | sed -e 's/uid=//; s/(.*//'`
if [ $ID -eq 0 ] ; then
  RUN_AS_ROOT=1
else
  DB2AUDIT_PATH=`cat /etc/passwd | grep "^$INSTANCE_NAME:" | sed -e "s/^$INSTANCE_NAME:[^:]*:[^:]*:[^:]*:[^:]*:\([^:]*\):[^:]*/\1/"`/sqllib/adm
  if [ ! $? -eq 0 ] ; then
   echo `date "+%a %D %T"` Error reading /etc/passwd file. Exiting. >> $ACTUATOR_LOG
   return 1
  fi
  DB2INSTANCE=$INSTANCE_NAME
  export DB2INSTANCE
fi


# internal variables
if [ $RUN_AS_ROOT -eq 1 ] ; then
 CURRENT_DIR=`su - $INSTANCE_NAME -c "pwd" | tail -1`
else
 CURRENT_DIR=`pwd`
fi
TEMP_DIR="$CURRENT_DIR/$PROPSFILE.dir"
DB_DISABLE=0
DATABASE_FC=1
ARCHLIST_IS_EMPTY=1
ALL_DB_ARCHLIST=$TEMP_DIR/all_db_archlist.txt
chmod 777 $TEMP_DIR

# checking whether database level archive path exists
if [ "$DATABASE_ARCHIVE_PATH" = "skip" ] ; then
 # disabling database
 echo `date "+%a %D %T"` Database level audit is disabled. >> $ACTUATOR_LOG
 touch $TEMPFILE4
 return 0
fi
if [ "$DATABASE_ARCHIVE_PATH" = "default" ] ; then
 if [ $RUN_AS_ROOT -eq 1 ] ; then
  DATABASE_ARCHIVE_PATH=`su - $INSTANCE_NAME -c 'db2audit describe | grep Archive | cut -d \" -f 2' | tail -1`
 else
  DATABASE_ARCHIVE_PATH=`$DB2AUDIT_PATH/db2audit describe | grep Archive | cut -d \" -f 2`
 fi
fi
if [ ! -d "$DATABASE_ARCHIVE_PATH" ] ; then
 # disabling database
 DB_DISABLE=1 
 echo `date "+%a %D %T"` Database level audit is disabled. >> $ACTUATOR_LOG
 touch $TEMPFILE4
 return 0
fi


if [ -f $TEMPFILE4 ] ; then
 rm -f $TEMPFILE4
fi

# checking if list of the archives exists
LIST=""
if [ -f $ALL_DB_ARCHLIST ] ; then 
 for i in `cat $ALL_DB_ARCHLIST` ; do
  ARCHLIST_IS_EMPTY=0
  echo `date "+%a %D %T"` Packing archive $i. >> $ACTUATOR_LOG
  LIST=$DATABASE_ARCHIVE_PATH/$i

 if [ $RUN_AS_ROOT -eq 1 ] ; then
      if [ ! -f $TEMPFILE4 ] ; then
        tar -cf $TEMPFILE4 "$LIST"
          if [ ! $? -eq 0 ] ; then
           echo `date "+%a %D %T"` Can not archive $i file. >>  $ACTUATOR_LOG
           echo `date "+%a %D %T"` Possible access error. >>  $ACTUATOR_LOG
          fi
      else
        tar -rf $TEMPFILE4 "$LIST"
         if [ ! $? -eq 0 ] ; then
           echo `date "+%a %D %T"` Can not archive $i file. >>  $ACTUATOR_LOG
           echo `date "+%a %D %T"` Possible access error. >>  $ACTUATOR_LOG
         fi
      fi  
 else
      if [ ! -f $TEMP_DIR/temp.tar ] ; then
         sudo -u $INSTANCE_NAME tar -cf $TEMP_DIR/temp.tar "$LIST"
           if [ ! $? -eq 0 ] ; then
            echo `date "+%a %D %T"` Can not archive $i file. >>  $ACTUATOR_LOG
            echo `date "+%a %D %T"` Possible wrong sudo settings or access error. >>  $ACTUATOR_LOG
           fi
      else
         sudo -u $INSTANCE_NAME tar -rf $TEMP_DIR/temp.tar "$LIST"
           if [ ! $? -eq 0 ] ; then
            echo `date "+%a %D %T"` Can not archive $i file. >>  $ACTUATOR_LOG
            echo `date "+%a %D %T"` Possible wrong sudo settings or access error. >>  $ACTUATOR_LOG
           fi
      fi
 fi
done

 if [ $RUN_AS_ROOT -eq 0 ] ; then
  mv $TEMP_DIR/temp.tar $TEMPFILE4
 fi

 if [ ! -f $TEMPFILE4 ] ; then
  touch $TEMPFILE4
 fi
 
 else
 echo `date "+%a %D %T"` Nothing to pack. Exiting. >> $ACTUATOR_LOG
 touch $TEMPFILE4
fi

if [ $ARCHLIST_IS_EMPTY -eq 1 ] ; then 
 echo `date "+%a %D %T"` Nothing to pack. Exiting. >> $ACTUATOR_LOG
 touch $TEMPFILE4
fi 

echo `date "+%a %D %T"` Collect of database level original logs successful. >> $ACTUATOR_LOG
rm -rf $TEMP_DIR

echo `date "+%a %D %T"` Collect of new events is successful. >> $ACTUATOR_LOG
return 0
