#!/bin/ksh
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

# input parameters
temp_file=$1
log_file=$2
log_type=$3
props_file=$4

# check for tail
if [ `uname` = SunOS ];
then
   TAIL='tail -1';
else
   TAIL='tail -n 1';
fi

if [ "$props_file" = "" ]; 
then
   echo "[`date`] ERROR: Missing property file argument">>$act_log_file;
   exit 1;
fi

#check property log temp file
if [ "$temp_file" = "" ];
then
   echo "[`date`] ERROR: Missing $log_type log output file argument">>$act_log_file;
   exit 1;
fi

#check property log path
if [ "$log_file" = "" ];
then
   echo "[`date`] WARNING: $log_type log path in ES properties is empty ">>$act_log_file;
   touch $temp_file
   exit 0;
fi   

# add filename to incoming path
log_file="$log_file/FBT_audit0.log"
#check whether log file path is correctly specified
if [ -f "$log_file" ];
then
   echo "[`date`] $log_type log file found" >>$act_log_file;
else
   echo "[`date`] ERROR: $log_type log file $log_file not found" >>$act_log_file;
   touch $temp_file
   exit 1
fi

iTempFile=$temp_file
# location of the log files
cDirName=`dirname $log_file`
# number of the log files
iNumberFiles=`ls -l $cDirName/FBT_audit*.log | wc -l | tr -d " "`
# the number of last line collected 
iLineNum=0
# base name of the log file
cBaseName=""
# the last number of the log file
iLastFileNumber=$((iNumberFiles-1))
# if there are no files in the directory
if [ $iLastFileNumber -lt 0 ];
then
   iLastFileNumber=0;
fi

echo "[`date`] Reading actuator state information" >>$act_log_file

#reading actuator state information from props file
if [ -f "$props_file" ];
then
   props_str1=`cat $props_file | head -n 1`;
   props_str2=`cat $props_file | head -n 2 | $TAIL | tr -d " "`;
   props_str3=`cat $props_file | head -n 3 | $TAIL`;
   props_str4=`cat $props_file | head -n 4 | $TAIL | tr -d " "`;

   # if it is runtime logs
   if [ $log_type = 'runtime' ]
   then
      # the first string of the file
      cFileStr=$props_str1;
      # the number of the last collect event 
      iLineNum=$props_str2;
   # if it is management logs
   else
      # the first string of the file
      cFileStr=$props_str3;
      # the number of the last collect event 
      iLineNum=$props_str4;
   fi

   # if the file string is empty
   if [ `echo $cFileStr | wc -m` -le 1  ]
   then
      echo "[`date`] Information about last collect of $log_type audit logs does not exist. This is initial collect">>$act_log_file
      # it is the initial collect - we shall collect two files FBT_audit0.log and FBT_audit1.log
      cBaseName="FBT_audit1.log"
      # in the FBT_audit1.log we starting from the first line
      iLineNum=0
      # the last file number
      iLastFileNumber=1
   else
      # filename of the last actual log file
      cLastFileName=`grep -l "$cFileStr" $cDirName/FBT_audit*.log`
      # if exists 
      if [ -f "$cLastFileName" ]
      then 
         echo "[`date`] $log_type audit file found - $cLastFileName">>$act_log_file 
         cBaseName=`basename $cLastFileName`
         iLastFileNumber=`basename $cBaseName .log | cut -c 10-`
      else
         echo "[`date`] WARNING: Full rollover of audit files since last collect occured - we miss some events. Please, collect more often">>$act_log_file
         echo "[`date`] Collecting all $log_type audit files from the directory">>$act_log_file
         # we shall collect all files from FBT_audit0.log to FBT_auditN.log
         cBaseName=`basename "$cDirName/FBT_audit$iLastFileNumber.log"`
         # in the FBT_auditN.log we starting from the first line
         iLineNum=0
      fi 
   fi

else
   echo "[`date`] Props file does not exist. This is initial collect">>$act_log_file
   # it is the initial collect - we shall collect two files FBT_audit0.log and FBT_audit1.log
   cBaseName="FBT_audit1.log"
   # in the FBT_audit1.log we starting from the first line
   iLineNum=0
   # the last file number
   iLastFileNumber=1
fi

bWrapAround=0
iWAC=0

while [ $bWrapAround -eq 0 ]; do

   # deleting temp file
   rm -f "$temp_file"

   # getting log rollover information 
   wrap_around_str0=`head -n 1 $cDirName/FBT_audit0.log | cut -d ' ' -f4`
   wrap_around_strL=`head -n 1 $cDirName/FBT_audit$((iNumberFiles-1)).log | cut -d ' ' -f4`

   # actuator state information
   new_file_str=""
   new_line_num=""

   # we starting from the last file number
   ifile=$iLastFileNumber

   while [ $ifile -ge 0 ]
   do
      # current filename
      cur_filename="$cDirName/FBT_audit$ifile.log"
      # checking if does file exist
      if [ -f "$cur_filename" ] 
      then 
         # if this file is FBT_audit0.log - ...
         if [ `echo "$cur_filename" | grep "FBT_audit0.log" | wc -m` -gt 0 ]
         then
            new_file_str=`head -n 1 $cur_filename | cut -d ' ' -f4`
            new_line_num=`wc -l $cur_filename | awk '{print $1}'`
            # if the FBT_audit0.log is empty
            if [ "$new_file_str" = "" ]
            then
               new_file_str=`head -n 1 $cDirName/FBT_audit1.log | cut -d ' ' -f4`
               new_line_num=`wc -l $cDirName/FBT_audit1.log | awk '{print $1}'`
            fi
         fi
         
         if [ `echo $cur_filename | grep "$cBaseName" | wc -m` -gt 0 ]
         then
            # the quantity of lines in current file 
            iQLines=`wc -l $cur_filename | awk '{print $1}'`
            # the quantity of string to collect from the end of file
            let iActLine=iQLines-iLineNum
            # collecting ... act_line from the end
            if [ `uname` = SunOS ];
            then
                tail -$iActLine $cur_filename >> $iTempFile;
                if [ $? -ne 0 ] 
                then 
                    echo "[`date`] ERROR occured during reading data from $cur_filename file or writing data to temp file">>$act_log_file
                    # deleting temp file
                    rm -f "$iTempFile"
                    touch $iTempFile
                    exit 1
            fi
            else
                tail -n $iActLine $cur_filename >> $iTempFile;
                if [ $? -ne 0 ] 
                then 
                    echo "[`date`] ERROR occured during reading data from $cur_filename file or writing data to temp file">>$act_log_file
                    # deleting temp file
                    rm -f "$iTempFile"
                    touch $iTempFile
                    exit 1
                fi
            fi
            echo "[`date`] Collect file $cur_filename from string N$(($iLineNum+1)) - $iActLine new events">>$act_log_file
        # this file is the last one we need to collect
         else
            # collecting ... all the file
            cat $cur_filename >> $iTempFile
            if [ $? -ne 0 ] 
            then 
                echo "[`date`] ERROR occured during reading data from $cur_filename file or writing data to temp file">>$act_log_file
                # deleting temp file
                rm -f "$iTempFile"
                touch $iTempFile
                exit 1
            fi
            echo "[`date`] Collected file $cur_filename - $((`wc -l $cur_filename | awk '{print $1}' | tr -d " "`)) new events ">>$act_log_file
         fi # last actual one
      else
         # the file does not exist
         let ifile=ifile-1
         continue
      fi # file exists
      let ifile=ifile-1
   done # for ifile

   bWrapAround=1
   b0WrapAround=1
   b1WrapAround=1

   # if the file FBT_audit0.log is not empty 
   if [ `echo $wrap_around_str0 | wc -m` -gt 1 ] 
   then
      # checking for log rollover first log file
      if [ `grep -l "$wrap_around_str0" $cDirName/FBT_audit0.log | wc -m` -eq 0 ] 
      then
         echo "[`date`] Check wrap-around [FBT_audit0.log] - ERROR">>$act_log_file
         echo "[`date`] Trying to collect again ...">>$act_log_file
         b0WrapAround=0
      else
         echo "[`date`] Check wrap-around [FBT_audit0.log] - OK">>$act_log_file
         b0WrapAround=1
      fi
   else
   echo "[`date`] WARNING file FBT_audit0.log - empty">>$act_log_file
      b0WrapAround=1
   fi

   # if the file FBT_auditX.log is not empty 
   if [ `echo $wrap_around_strL | wc -m` -gt 1 ] 
   then
      # checking for log rollover last log file
      if [ `grep -l "$wrap_around_strL" $cDirName/FBT_audit$((iNumberFiles-1)).log | wc -m` -eq 0 ] 
      then
         echo "[`date`] Check wrap-around [FBT_audit$((iNumberFiles-1)).log] - ERROR">>$act_log_file
         echo "[`date`] Trying to collect again ...">>$act_log_file
         b1WrapAround=0
      else
         echo "[`date`] Check wrap-around [FBT_audit$((iNumberFiles-1)).log] - OK">>$act_log_file
         bWrapAround=1
      fi
   else
      echo "[`date`] WARNING file FBT_audit$((iNumberFiles-1)).log - empty">>$act_log_file
      b0WrapAround=1
   fi
      
   if [ b0WrapAround -eq 0 ]
   then
      bWrapAround=0
   fi
      
   if [ b1WrapAround -eq 0 ]
   then
      bWrapAround=0
   fi
      
   if [ $iWAC -gt 100 ]
   then
      echo "[`date`] ERROR - to many log files rollover ">>$act_log_file
      exit 1
   fi
   
   let iWAC=iWAC+1

   # if rollover occures - we check how many files are in current directory
   if [ bWrapAround -eq 0 ]
   then
      # number of the log files
      iNumberFiles=`ls -l $cDirName/FBT_audit*.log | wc -l | tr -d " "`
      # the last number of the log file
      iLastFileNumber=$((iNumberFiles-1))   
      
      # if the file string is empty
      if [ `echo $cFileStr | wc -m` -le 1  ]
      then
         # it is the initial collect - we shall collect two files FBT_audit0.log and FBT_audit1.log
         cBaseName="FBT_audit1.log"
         # in the FBT_audit1.log we starting from the first line
         iLineNum=0
         # the last file number
         iLastFileNumber=1
      else
         # filename of the last actual log file
         cLastFileName=`grep -l "$cFileStr" $cDirName/FBT_audit*.log`
         # if exists 
         if [ -f "$cLastFileName" ]
         then 
            cBaseName=`basename $cLastFileName`
            iLastFileNumber=`basename $cBaseName .log | cut -c 10-`
         else
            # we shall collect all files from FBT_audit0.log to FBT_auditN.log
            cBaseName=`basename "$cDirName/FBT_audit$iLastFileNumber.log"`
            # in the FBT_auditN.log we starting from the first line
            iLineNum=0
         fi 
      fi
   fi   

done # while wrap-around

echo "[`date`] Writing actuator state information">>$act_log_file

# if it is runtime logs
if [ $log_type = 'runtime' ]
then
   props_str1=$new_file_str;
   props_str2=$new_line_num;
# if it is management logs
else
   props_str3=$new_file_str;
   props_str4=$new_line_num;
fi

echo "$props_str1">$props_file
echo "$props_str2">>$props_file
echo "$props_str3">>$props_file
echo "$props_str4">>$props_file

exit 0
