REM Licensed Materials - Property of IBM
REM 5724-S67
REM (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
REM US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
@echo off
rem ################################getitfimlogs.bat######################################
rem Receive parameters from the TCIM actuator
rem ######################################################################################
set RTEMP_FILE=%1
set MTEMP_FILE=%2
set RLOG_PATH=%3
for /F "delims=" %%i in ('@echo %RLOG_PATH%') do (set RLOG_PATH=%%~fi\)
set MLOG_PATH=%4
for /F "delims=" %%i in ('@echo %MLOG_PATH%') do (set MLOG_PATH=%%~fi\)
set PROPS_FILE=%5
set TEMP_LOG="..\log\getitfimlogs.err"

rem ######################################################################################
rem TCIM actuator parameters check
rem ######################################################################################

if "%PROPS_FILE%" EQU "" (
  rem -- clear temp log file
  type nul> %TEMP_LOG%
  @echo [%date%][%time%] "ERROR: Missing property file argument">> %TEMP_LOG%
  @echo [%date%][%time%] "FAILURE: Processing finished [Return code 1]">> %TEMP_LOG%
  call :cleanup
  exit /b 1
)

rem -- clear log file
type nul> ..\log\%PROPS_FILE%.log

@echo [%date%][%time%] "Processing started ...">> ..\log\%PROPS_FILE%.log

if "%RTEMP_FILE%" EQU "" (
  @echo [%date%][%time%] "ERROR: Missing output file argument for runtime logs">> ..\log\%PROPS_FILE%.log
  @echo [%date%][%time%] "FAILURE: Processing finished [Return code 1]">> ..\log\%PROPS_FILE%.log
  call :cleanup
  exit /b 1
)

if "%MTEMP_FILE%" EQU "" (
  @echo [%date%][%time%] "ERROR: Missing output file argument for management logs">> ..\log\%PROPS_FILE%.log
  @echo [%date%][%time%] "FAILURE: Processing finished [Return code 1]">> ..\log\%PROPS_FILE%.log
  call :cleanup
  exit /b 1
)

if "%RLOG_PATH%" EQU "" (
  type nul> "%RTEMP_FILE%"
)

if "%MLOG_PATH%" EQU "" (
  type nul> "%MTEMP_FILE%"
)

rem -- create temp directory
if exist %PROPS_FILE%.dir (
  rmdir /S /Q %PROPS_FILE%.dir
)
mkdir %PROPS_FILE%.dir


rem ######################################################################################
rem variables for runtime and management logs
rem ######################################################################################
rem -- runtime logs
:runtime_setup
set LEVEL=mgmt_setup
set LOG_CATEGORY=runtime
set LOG_PATH=%RLOG_PATH%
set TEMP_FILE=%RTEMP_FILE%
set /A LOG_FAILURE=0
set BOOL_LOG=true
set /A CHECK_COUNT=0
set /A CHECK_WRAP=0
@echo [%date%][%time%] "##########################################">> ..\log\%PROPS_FILE%.log
@echo [%date%][%time%] "###    Processing %LOG_CATEGORY% log files    ###">> ..\log\%PROPS_FILE%.log
@echo [%date%][%time%] "##########################################">> ..\log\%PROPS_FILE%.log
call :check_path
if errorlevel 1 ( goto path_failure ) else ( goto save_logdirs_tree )

:mgmt_setup
set LEVEL=eof
set LOG_CATEGORY=mgmt
set LOG_PATH=%MLOG_PATH%
set TEMP_FILE=%MTEMP_FILE%
set BOOL_LOG=true
set /A CHECK_COUNT=0
set /A CHECK_WRAP=0
@echo [%date%][%time%] "#######################################">> ..\log\%PROPS_FILE%.log
@echo [%date%][%time%] "###    Processing %LOG_CATEGORY% log files    ###">> ..\log\%PROPS_FILE%.log
@echo [%date%][%time%] "#######################################">> ..\log\%PROPS_FILE%.log
call :check_path
if errorlevel 1 ( goto path_failure ) else ( goto save_logdirs_tree )

rem ######################################################################################
rem print error message about path being empty
rem ######################################################################################
:path_failure
@echo [%date%][%time%] "WARNING: Path for %LOG_CATEGORY% audit log files in ES properties is empty">> ..\log\%PROPS_FILE%.log
set /A LOG_FAILURE+=1
goto %LEVEL%

rem ######################################################################################
rem save current state of log directories and count log files
rem ######################################################################################
rem -- save current state of log dirs
:save_logdirs_tree
if exist "%LOG_PATH%" (
  call :check_path
  if errorlevel 1 (
    set /A LOG_FAILURE+=1
	goto %LEVEL%
  )
  if exist "%LOG_PATH%FBT_audit0.log" (
    for /F "delims=" %%i in ('@echo %LOG_PATH%FBT_audit0.log') do (set LOG_PATH=%%~dpi)
    @echo [%date%][%time%] "%LOG_CATEGORY% audit log files are found in %LOG_PATH%">> ..\log\%PROPS_FILE%.log
    for %%i in ("%LOG_PATH%FBT_audit0.log") do (
      if %%~zi EQU 0 (
        @echo [%date%][%time%] "WARNING: %LOG_CATEGORY% audit log file FBT_audit0.log is empty">> ..\log\%PROPS_FILE%.log
        type nul> "%TEMP_FILE%"
        set BOOL_LOG=false
        set /A LOG_FAILURE+=1
        goto %LEVEL%
      ) else (
          set /A LOG_COUNT=0
          @echo [%date%][%time%] "%LOG_CATEGORY% audit log file FBT_audit0.log contains records">> ..\log\%PROPS_FILE%.log
          for /F "delims=" %%i in ('dir /A-D /B /ON "%LOG_PATH%*.log"') do (
            dir /A-D /B /ON "%LOG_PATH%" | findstr /R /C:"^FBT_audit[0-9]*\.log$"> %PROPS_FILE%.dir\temp.lst
            set /A LOG_COUNT+=1
          )
      )
    )
    if exist %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%.lst (
      type nul> %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%.lst
    )
    for /F "delims=" %%j in ('type "%PROPS_FILE%.dir\temp.lst"') do (
      @echo %LOG_PATH%%%j>> %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%.lst
    )
  ) else (
      @echo [%date%][%time%] "WARNING: There are no %LOG_CATEGORY% audit log files">> ..\log\%PROPS_FILE%.log
      type nul> %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%_before.props
      type nul> %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%_after.props
      type nul> "%TEMP_FILE%"
      set BOOL_LOG=false
      set /A LOG_FAILURE+=1
      goto %LEVEL%
  )
) else (
    @echo [%date%][%time%] "ERROR: Path %LOG_PATH% for %LOG_CATEGORY% audit log files cannot be found">> ..\log\%PROPS_FILE%.log
    type nul> "%TEMP_FILE%"
    set BOOL_LOG=false
    set /A LOG_FAILURE+=1
    if exist %PROPS_FILE%.dir\*_after.props (
      copy /Y %PROPS_FILE%.dir\*_after.props %PROPS_FILE%> nul
    )
    goto %LEVEL%
)


rem ######################################################################################
rem read globalID of the first event from the active(0)and last(N)log files
rem ######################################################################################
:save_globalids
rem -- save line number of the last event in FBT_audit0.log
set /A %LOG_CATEGORY%_LINE_NUM=0
set /A TEST_LINE_NUM=0
for /F "delims=" %%a in ('type "%LOG_PATH%FBT_audit0.log"') do (
  set /A %LOG_CATEGORY%_LINE_NUM+=1
)
set /A TEST_LINE_NUM=%LOG_CATEGORY%_LINE_NUM
for /F "tokens=1-4" %%i in ('type "%LOG_PATH%FBT_audit0.log"') do (
  for /F "tokens=1,2 delims==" %%m in ('@echo %%l') do (
    @echo %%n> %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%.sav
    @echo %LOG_CATEGORY%_FILE_STR=%%n> %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%_after.props
    @echo %LOG_CATEGORY%_LINE_NUM=%TEST_LINE_NUM% >> %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%_after.props
    set /A TEST_LINE_NUM=0
    goto save_globalids2
  )
)
:save_globalids2
if %LOG_COUNT% GTR 1 (
  set /A MAX_LOG=%LOG_COUNT%-1
) else (
    set /A MAX_LOG=0
)
@echo [%date%][%time%] "The last %LOG_CATEGORY% log file is FBT_audit%MAX_LOG%.log">> ..\log\%PROPS_FILE%.log
for /F "tokens=1-4" %%i in ('type "%LOG_PATH%FBT_audit%MAX_LOG%.log"') do (
  for /F "tokens=1,2 delims==" %%m in ('@echo %%l') do (
    @echo %%n>> %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%.sav
    if exist %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%.sav (
      copy /Y %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%.sav %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%.chk> nul
    )
    goto read_state
  )
)

rem ######################################################################################
rem read state information from props file
rem ######################################################################################
:read_state
@echo [%date%][%time%] "Reading state information of %LOG_CATEGORY% logs">> ..\log\%PROPS_FILE%.log
type nul > %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%_before.props
if exist %PROPS_FILE% (
  findstr "%LOG_CATEGORY%" %PROPS_FILE%> nul
  if NOT errorlevel 1 (
    type nul > %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%_before.props
    for /F "delims=" %%i in ('findstr "%LOG_CATEGORY%" %PROPS_FILE%') do (
      @echo %%i >> %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%_before.props
    )
    for /F "tokens=2 delims==" %%i in ('findstr "%LOG_CATEGORY%_FILE_STR" %PROPS_FILE%') do (set %LOG_CATEGORY%_FILE_STR=%%i)
    for /F "tokens=2 delims==" %%i in ('findstr "%LOG_CATEGORY%_LINE_NUM" %PROPS_FILE%') do (set /A %LOG_CATEGORY%_LINE_NUM=%%i)
    set BOOL_FIRST_COLLECT=false
  ) else (
      @echo [%date%][%time%] "This is the first collect">> ..\log\%PROPS_FILE%.log
      @echo [%date%][%time%] "Collecting only 2 %LOG_CATEGORY% log files">> ..\log\%PROPS_FILE%.log
      set BOOL_FIRST_COLLECT=true
  )
) else (
    @echo [%date%][%time%] "This is the first collect">> ..\log\%PROPS_FILE%.log
    @echo [%date%][%time%] "Collecting only 2 %LOG_CATEGORY% log files">> ..\log\%PROPS_FILE%.log
    set BOOL_FIRST_COLLECT=true
)

rem ######################################################################################
rem collect logs
rem ######################################################################################
:run_collect
set FILE_TO_START=
if "%BOOL_FIRST_COLLECT%" == "true" (
  set /A MAX_LOG=1
  set /A TEST_LINE_NUM=0
) else (
    set /A TEST_LINE_NUM=%LOG_CATEGORY%_LINE_NUM
    if "%LOG_CATEGORY%" == "runtime" (
      set TEMP_FILE_STR=%runtime_FILE_STR%
    ) else (
        set TEMP_FILE_STR=%mgmt_FILE_STR%
    )
)
rem -- if props file contains the line num from previous collect
rem -- find the file that holds the last collected event 
if %TEST_LINE_NUM% GTR 0 (
  for /F "delims=" %%i in ('findstr /M /F:"%PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%.lst" /L "%TEMP_FILE_STR%"') do (
    set FILE_TO_START=%%i
  )
) else (
    for %%i in ("%TEMP_FILE%") do (
      if %%~zi GTR 0 (
        type nul> "%TEMP_FILE%"
      )
    )
    for /L %%i in (%MAX_LOG%,-1,0) do (
      if exist "%LOG_PATH%FBT_audit%%i.log" (
        type "%LOG_PATH%FBT_audit%%i.log">> "%TEMP_FILE%"
      )
    )
    if "%LEVEL%" == "check_wraparound" (goto check_wraparound) else (goto check)
)
rem -- collect from the last collected file strarting from the line in props file
if NOT "%FILE_TO_START%" == "" (
  for %%i in ("%TEMP_FILE%") do (
    if %%~zi GTR 0 (
      type nul> "%TEMP_FILE%"
    )
  )
  @echo [%date%][%time%] "Last collected record of %LOG_CATEGORY% log files was found in %FILE_TO_START% at line %TEST_LINE_NUM%">> ..\log\%PROPS_FILE%.log
  for /F "skip=%TEST_LINE_NUM% delims=" %%i in ('type "%FILE_TO_START%"') do (
    if NOT "%%i" == "" (
      @echo %%i>> "%TEMP_FILE%"
    )
  )
  for /F "delims=" %%i in ('@echo "%FILE_TO_START%"') do (set FILE_TO_START=%%~nxi)
  findstr /R "." "%TEMP_FILE%"> nul
  if errorlevel 1 (
    @echo [%date%][%time%] "No new records of %LOG_CATEGORY% log files were found in %FILE_TO_START% after line %TEST_LINE_NUM%">> ..\log\%PROPS_FILE%.log
    type nul> "%TEMP_FILE%"
  ) else (
      @echo [%date%][%time%] "All new records of %LOG_CATEGORY% log files in %FILE_TO_START% after line %TEST_LINE_NUM% were collected">> ..\log\%PROPS_FILE%.log
  )
) else (
    @echo [%date%][%time%] "WARNING: Full rollover of %LOG_CATEGORY% audit log files since last collect occured.">> ..\log\%PROPS_FILE%.log
    @echo [%date%][%time%] "Many %LOG_CATEGORY% events were lost and not collected - please collect more often.">> ..\log\%PROPS_FILE%.log
    @echo [%date%][%time%] "Collecting all present records of %LOG_CATEGORY% log files.">> ..\log\%PROPS_FILE%.log
    for %%i in ("%TEMP_FILE%") do (
      if %%~zi GTR 0 (
        type nul> "%TEMP_FILE%"
      )
    )
    goto collect_all
)
rem -- extract the number from the log file to collect everything from last collected log -1
for /F "tokens=1-3 delims=t." %%i in ('@echo "%FILE_TO_START%"') do (
  set /A MAX_LOG=%%j-1
)

:collect_all
rem -- collect everything
for /L %%i in (%MAX_LOG%,-1,0) do (
  if exist "%LOG_PATH%FBT_audit%%i.log" (
    type "%LOG_PATH%FBT_audit%%i.log">> "%TEMP_FILE%"
    @echo [%date%][%time%] "All records of %LOG_CATEGORY% log files in FBT_audit%%i.log were collected">> ..\log\%PROPS_FILE%.log
  )
)
@echo [%date%][%time%] "Collect of %LOG_CATEGORY% log files is finished">> ..\log\%PROPS_FILE%.log

if "%LEVEL%" == "check_wraparound" (goto check_wraparound) else (goto check)

rem -- check if wraparound occured during the collect
:check
@echo [%date%][%time%] "Checking wraparound of %LOG_CATEGORY% log files...">> ..\log\%PROPS_FILE%.log
set LEVEL=check_wraparound
goto save_logdirs_tree

:check_wraparound
set /A CHECK_COUNT+=1
set /A COUNT_DIFFS=0
set /A CHECK_WRAP+=1
if %CHECK_COUNT% LEQ 100 (
  for /F "delims=" %%i in ('fc /N %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%.sav %PROPS_FILE%.dir\%PROPS_FILE%_%LOG_CATEGORY%.chk') do (
    for /F "delims=" %%j in ('@echo %%i') do (set /A COUNT_DIFFS+=1)
  )
  @echo [%date%][%time%] "Wraparound of %LOG_CATEGORY% log files was checked %CHECK_WRAP% time(s)...">> ..\log\%PROPS_FILE%.log
) else (
    @echo [%date%][%time%] "ERROR: Too many (%CHECK_WRAP%) wraparounds of %LOG_CATEGORY% logs during the collect">> ..\log\%PROPS_FILE%.log
    if "%LOG_CATEGORY%" == "runtime" (
      del /F/Q %TEMP_FILE%
      goto mgmt_setup
    ) else (
        @echo [%date%][%time%] "FAILURE: Processing finished [Return code 1]">> ..\log\%PROPS_FILE%.log
        del /F/Q %TEMP_FILE%
        call :cleanup
        exit /b 1
    )
)
if %COUNT_DIFFS% GTR 2 (
  goto save_logdirs_tree
) else (
    @echo [%date%][%time%] "No wraparound occured during the %LOG_CATEGORY% logs collect">> ..\log\%PROPS_FILE%.log
)

if "%LOG_CATEGORY%" == "runtime" (
  goto mgmt_setup
) else (
    copy /Y %PROPS_FILE%.dir\*_after.props %PROPS_FILE%> nul
    goto eof
)

:eof
if exist %PROPS_FILE%.dir\*_after.props (
  copy /Y %PROPS_FILE%.dir\*_after.props %PROPS_FILE%> nul
)
if %LOG_FAILURE% GEQ 2 (
  @echo [%date%][%time%] "FAILURE: Processing finished [Return code 1]">> ..\log\%PROPS_FILE%.log
  call :cleanup
  exit /b 1
) else (
    @echo [%date%][%time%] "SUCCESS: Processing finished [Return code 0]">> ..\log\%PROPS_FILE%.log
    call :cleanup
    exit /b 0
)

:check_path
@echo %LOG_PATH% | findstr /R "^[A-Z]:"> nul
goto :EOF


rem ######################################################################################
rem cleanup routine
rem ######################################################################################
:cleanup
rem -- delete temp directory
if exist %PROPS_FILE%.dir (
  rmdir /S /Q %PROPS_FILE%.dir
)
if exist %TEMP_LOG% (
  for %%i in (%TEMP_LOG%) do (
    if %%~zi EQU 0 (
      del /F/Q %TEMP_LOG%
    )
  )
)
goto :EOF