REM Licensed Materials - Property of IBM
REM 5724-S67
REM (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
REM US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
@echo off

rem ####################################getsapnetweaverlogs.bat####################################
rem Receives parameters from the TCIM actuator
rem ######################################################################################
set ATEMP_FILE=%1
set ALOG_PATH=%2
set PROPS_FILE=%3
set TEMP_LOG=..\log\getsapnetweaverlogs.err

rem ######################################################################################
rem TCIM actuator parameters check
rem ######################################################################################
set PRINTING=enabled

rem -- Property file %PROPS_FILE% argument is missing
if "%PROPS_FILE%" EQU "" (
  rem -- clears temp log file
  type nul> "%TEMP_LOG%"
  call :print_message "ERROR: Missing property file argument" "%TEMP_LOG%"
  call :print_message "FAILURE: Processing finished [Return code 1]" "%TEMP_LOG%"
  call :cleanup
  exit /b 1
) else (
    rem -- sets actuator log name
    set ACTUATOR_LOG=..\log\%PROPS_FILE%.log
)

rem -- clears actuator log file and start logging
type nul> "%ACTUATOR_LOG%"
call :print_message "Processing started ..." "%ACTUATOR_LOG%"

rem -- Temporary file %TEMP_FILE% argument is missing
if "%ATEMP_FILE%" EQU "" (
  call :print_message "ERROR: Missing output file argument for SAP security audit logs" "%ACTUATOR_LOG%"
  goto failure
)

rem -- creates temp directory
if exist %PROPS_FILE%.dir (
  rmdir /S /Q %PROPS_FILE%.dir>> nul 2>&1
)
mkdir %PROPS_FILE%.dir>> nul 2>&1

rem ######################################################################################
rem variables for SAP log files
rem if new categories have to be added, 
rem new set of variables needs to be declared 
rem ######################################################################################
:audit_setup
set NEXT_LABEL=eof
set LOG_CATEGORY=AUDIT
set LOG_DESCRIPTION=SAP security audit
set /A LOG_NUM=0
set TIMESTAMP_LENGTH=8
call :print_message "Processing %LOG_DESCRIPTION% log files" "%ACTUATOR_LOG%"
set LOG_PATH=%ALOG_PATH%
set TEMP_FILE=%ATEMP_FILE%
set PRINTING=enabled
set /A LOG_FAILURE=0
call :path_value_check
if defined FAILURE (goto %FAILURE%) else (goto save_logs_list)

rem ######################################################################################
rem saving current state of logs directoriy routine
rem ######################################################################################
rem -- save current state of logs dir
:save_logs_list
(set FAILURE=)
if exist "%LOG_PATH%" (
  rem -- filters out all dirs and files which do not contain timestamp (YYYYMMDD) in their names
  for /F "delims=" %%i in ('dir /A-D /B /O-N %LOG_PATH%') do (
    @echo %%i | findstr /R "[2][0-1][0-9][0-9][0-1][0-9][0-3][0-9]"> nul
    if NOT errorlevel 1 (
      @echo %%i>> %PROPS_FILE%.dir\temp_list.lst
    )
  )
  rem -- calculates today's (param 0) date (YYYYMMDD), value is stored in %DAYSAGO%
  call :find_date "0"
  if errorlevel 1 (
    goto aux_failure
  ) else (
      for /F "delims=" %%i in ('type "%PROPS_FILE%.dir\temp_list.lst"') do (
        <nul (set /P string=%%i)> %PROPS_FILE%.dir\string.len
        for %%j in ("%PROPS_FILE%.dir\string.len") do (
          set /A STRING_LENGTH=%%~zj-%TIMESTAMP_LENGTH%
        )
        call :find_timestamp "%%STRING_LENGTH%%" "%%i">nul 2>&1
        if NOT errorlevel 1 (
          call :compare_vars "%%TIMESTAMP%%" "LEQ" "%%DAYSAGO%%">nul 2>&1
          if NOT errorlevel 1 (@echo %%i>> %PROPS_FILE%.dir\%LOG_CATEGORY%.lst)
        )
      )
      for %%i in ("%PROPS_FILE%.dir\%LOG_CATEGORY%.lst") do (
        if %%~zi EQU 0 (
          goto logs_not_found
        ) else (
            call :print_message "%LOG_DESCRIPTION% log files found in '%LOG_PATH%'" "%ACTUATOR_LOG%"
            if exist %PROPS_FILE% (
              type %PROPS_FILE% | findstr "%LOG_CATEGORY%"> nul 2>&1
              if errorlevel 1 (goto initial_collect) else (goto post_initial_collect)
            ) else (
                goto initial_collect
            )
        )
      )
  )
) else (
    goto logs_not_found
)

rem ######################################################################################
rem timestamp finding routine
rem finds a timestamp (YYYYMMDD) in the filename
rem takes two params: %1 - length of the string, %2 - string to search
rem ######################################################################################
:find_timestamp
set TEMP_STRING=%~2
for /L %%i in (0,1,%~1) do (
  call set /A TIMESTAMP=%%TEMP_STRING:~%%i,%TIMESTAMP_LENGTH%%%
  call :compare_vars "%%TIMESTAMP%%" "GTR" "0">nul 2>&1
  if NOT errorlevel 1 (exit /b 0 && goto :EOF)
)
goto :EOF

rem ######################################################################################
rem date calculating routine
rem calculates the date in format of YYYYMMDD for log files
rem takes a param: %1 - number of days ago (0=Today)
rem ######################################################################################
:find_date
set AUXILARY=..\bin\date.exe
call %AUXILARY% -d "%~1 days ago" +"%%Y%%m%%d">nul 2>&1 || exit /b 1 && goto :EOF
for /F %%i in ('..\bin\date.exe -d "%~1 days ago" +"%%Y%%m%%d"') do (
  set DAYSAGO=%%i
) 
goto :EOF

rem ######################################################################################
rem auxilary tools executing routine
rem executes aux utilities used in the script: smalldd.exe and bigchop.exe
rem takes 5 param: %1 - utility to execute, %2-%5 - aux utility specific params
rem ######################################################################################
:execute_aux
set AUXILARY=%~1
call %AUXILARY% %2 %3 %4 %5>nul 2>&1 || exit /b 1 && goto :EOF
goto :EOF

rem -- prints a message to the actuator log about inability to execute an auxillary utility
rem -- if some error(s) occure while executing (read/write failure)
:aux_failure
call :print_message "ERROR: execution of auxilary program '%AUXILARY%' failed" "%ACTUATOR_LOG%"
call :print_message "Please, perform the collect procedure again" "%ACTUATOR_LOG%"
set /A LOG_FAILURE+=1
goto %NEXT_LABEL%

rem ######################################################################################
rem vars comparing routine
rem compare 2 temp variables
rem takes 2 params: %1 - temprary variable 1, %2 - EQU, NEQ, LSS, LEQ, GTR, GEQ operands, %3 - temprary variable 2
rem ######################################################################################
:compare_vars
if %~1 %~2 %~3 (exit /b 0) else (exit /b 1)
goto :EOF

rem ######################################################################################
rem props file preparing routine
rem reads the name of today's audit log file (or closest available) and its size
rem takes a param: %1 - name of last file collected
rem ######################################################################################
:prepare_props
set LAST_FILE=%~1
for %%i in ("%LOG_PATH%%LAST_FILE%") do (set /A LAST_BYTE=%%~zi)
goto :EOF

rem ######################################################################################
rem initial collect routine
rem collects log files for today and 2 previous days
rem creates props file
rem ######################################################################################
:initial_collect
call :print_message "This is the initial collect" "%ACTUATOR_LOG%"
call :print_message "Collecting only events from %LOG_DESCRIPTION% log files for today and 2 previous days" "%ACTUATOR_LOG%"
rem -- calculates 2 days ago date in format of YYYYMMDD, value is stored in %DAYSAGO%
call :find_date "2"
call :prepare_copy_list "GEQ"
call :execute_copy_command "INITIAL"
if defined FAILURE (
  goto %FAILURE%
) else (
    call :print_message "All records of %LOG_DESCRIPTION% log files were collected" "%ACTUATOR_LOG%"
    goto %NEXT_LABEL%
)

rem ######################################################################################
rem prepare copy command routine
rem creates additional batch file with command copy /Y /B [source1]+...+/B [sourceN] /B [target]
rem ######################################################################################
:prepare_copy_command
set FILE_TO_CHOP=%PROPS_FILE%.dir\%LOG_CATEGORY%temp.chunk
call :compare_vars "%%LOG_NUM%%" "GTR" "0">nul 2>&1
if errorlevel 1 (goto :EOF)
if exist "%PROPS_FILE%.dir\%LOG_CATEGORY%.chunk" (
  <nul (set /P z=copy /Y /B "%PROPS_FILE%.dir\%LOG_CATEGORY%.chunk"+) >> %PROPS_FILE%.dir\%LOG_CATEGORY%_copy.bat
) else (
    <nul (set /P z=copy /Y ) >> %PROPS_FILE%.dir\%LOG_CATEGORY%_copy.bat
)
for /F "delims=" %%i in ('type %1') do (
  set /A TEMP+=1
  call :compare_vars "%%TEMP%%" "EQU" "%%LOG_NUM%%">nul 2>&1
  if errorlevel 1 (<nul (set /P z="/B "%LOG_PATH%%%i"+") >> %PROPS_FILE%.dir\%LOG_CATEGORY%_copy.bat) else (<nul (set /P z="/B "%LOG_PATH%%%i" /B "%FILE_TO_CHOP%" > nul") >> %PROPS_FILE%.dir\%LOG_CATEGORY%_copy.bat)
)
goto :EOF

rem ######################################################################################
rem executing 'copy' command batch script routine
rem updates props file
rem uses external utility to chop collected file such that 
rem final %TEMP_FILE% has every record (200 bytes) followed by CRLF (new line)
rem takes a param: %1 - type of collect:INITIAL, POST_INITIAL
rem ######################################################################################
:execute_copy_command
set /A BYTES_TO_CHOP=200
if exist "%PROPS_FILE%.dir\%LOG_CATEGORY%_copy.bat" (
  call "%PROPS_FILE%.dir\%LOG_CATEGORY%_copy.bat"> nul || (call :print_message "ERROR: %LOG_DESCRIPTION% log files read/write failure" "%ACTUATOR_LOG%" & set FAILURE=failure)
  call :write_props "%LAST_FILE%" "%LAST_BYTE%" "%PROPS_FILE%.dir\%LOG_CATEGORY%.props"
  call :execute_aux "..\bin\bigchop.exe" "%BYTES_TO_CHOP%" "%FILE_TO_CHOP%" "%TEMP_FILE%" "DOS"
  if errorlevel 1 (
    del /F/Q %PROPS_FILE%.dir\%LOG_CATEGORY%.props && set FAILURE=aux_failure && goto :EOF
  ) else (
      if exist "%PROPS_FILE%.dir\%LOG_CATEGORY%.cpy" (
        for /F "delims=" %%i in ('type "%PROPS_FILE%.dir\%LOG_CATEGORY%.cpy"') do (
          call :print_message "Collecting new records from '%LOG_PATH%%%i'...Ok" "%ACTUATOR_LOG%"
        )
      )
  )
) else (
    if %1 == "INITIAL" (
     (set FAILURE=initial_failure)
    ) else (
        (set FAILURE=post_initial_failure)
    )
)
goto :EOF

rem ######################################################################################
rem preparing 'copy' command list of files routine
rem checks and adds the names of files to a temporary list that is used to feed 'copy' command
rem names (first 8 digits) are compared to the value stored in %DAYSAGO%
rem takes a param: %1 - EQU, NEQ, LSS, LEQ, GTR, GEQ operands
rem ######################################################################################
:prepare_copy_list
for /F "delims=" %%i in ('type "%PROPS_FILE%.dir\%LOG_CATEGORY%.lst"') do (
  <nul (set /P string=%%i)> %PROPS_FILE%.dir\string.len
  for %%j in ("%PROPS_FILE%.dir\string.len") do (
    set /A STRING_LENGTH=%%~zj-%TIMESTAMP_LENGTH%
  )
  call :find_timestamp "%%STRING_LENGTH%%" "%%i">nul 2>&1
  if NOT errorlevel 1 (
    call :compare_vars "%%TIMESTAMP%%" "%~1" "%%DAYSAGO%%">nul 2>&1
    rem -- stores all filenames with their timestamps with * delimiter to the temporary copy list that meet the condition above
    if NOT errorlevel 1 (call echo %%TIMESTAMP%%^*%%i>> %PROPS_FILE%.dir\%LOG_CATEGORY%.tmp && set /A LOG_NUM+=1)
  )
)
rem -- sort the temp list such that the files with farthest timestamp come first
type "%PROPS_FILE%.dir\%LOG_CATEGORY%.tmp" | sort> %PROPS_FILE%.dir\%LOG_CATEGORY%_timestamps.cpy
rem -- finalizes the list such that the only filenames are stored in chronological order (with farthest timestamp from today - first)
for /F "tokens=1,2 delims=*" %%i in ('type "%PROPS_FILE%.dir\%LOG_CATEGORY%_timestamps.cpy"') do (
  @echo %%j>> %PROPS_FILE%.dir\%LOG_CATEGORY%.cpy
  call :prepare_props "%%j"
)
call :prepare_copy_command "%PROPS_FILE%.dir\%LOG_CATEGORY%.cpy"
goto :EOF

rem -- prints a message to the actuator log about no new records being found 
rem -- if no files are found for today and two previous days 
:initial_failure
call :print_message "WARNING: %LOG_DESCRIPTION% log files for today and 2 previous days not found in %LOG_PATH%" "%ACTUATOR_LOG%"
call :print_message "Configure %LOG_DESCRIPTION% log, Transaction SM19, in order for SAP to start logging events and repeat the collect" "%ACTUATOR_LOG%"
if exist "%PROPS_FILE%.dir\%LOG_CATEGORY%.lst" (
  call :write_props "%LAST_FILE%" "%LAST_BYTE%" "%PROPS_FILE%.dir\%LOG_CATEGORY%.props"
) else (
    call :write_props "%DAYSAGO%.AUD" "0" "%PROPS_FILE%.dir\%LOG_CATEGORY%.props"
)
type nul> "%TEMP_FILE%"
goto %NEXT_LABEL%

rem ######################################################################################
rem post initial collect routine
rem collects log files starting from last collected untill today (included)
rem updates props file
rem ######################################################################################
:post_initial_collect
call :print_message "Reading props file '%PROPS_FILE%' to get the information about previous collect..." "%ACTUATOR_LOG%"
rem -- reads the name of last collected file and its size from props file
for /F "tokens=1-3 delims=*" %%i in ('type "%PROPS_FILE%"') do (
  set LAST_FILE_PROPS=%%j
  set /A LAST_BYTE_PROPS=%%k
) || ( set FAILURE=props_failure )
if exist "%LOG_PATH%%LAST_FILE_PROPS%" (
  rem -- finds out actual size of last collected file stored in props file
  for %%i in ("%LOG_PATH%%LAST_FILE_PROPS%") do (set /A CURRENT_BYTE=%%~zi)
  call :compare_vars "%%LAST_BYTE_PROPS%%" "LSS" "%%CURRENT_BYTE%%">nul 2>&1
  if NOT errorlevel 1 (
    rem -- reads bytes from the position+1 to eof of the last collected file stored in props file
    rem -- stores temporary file with data (new events)
    set /A LAST_BYTE_PROPS+=1
    call :execute_aux "..\bin\smalldd.exe" "%%LAST_BYTE_PROPS%%" "EOF" "%LOG_PATH%%LAST_FILE_PROPS%" "%PROPS_FILE%.dir\%LOG_CATEGORY%.chunk"
    if errorlevel 1 (
      goto aux_failure
    ) else (
        call :print_message "New %LOG_DESCRIPTION% log records found in '%LOG_PATH%'" "%ACTUATOR_LOG%"
        call :print_message "Collecting new records from '%LOG_PATH%%LAST_FILE_PROPS%'...Ok" "%ACTUATOR_LOG%"
    )
  )
) else (
    if %LAST_BYTE_PROPS% GTR 0 (
      rem -- last collected file stored in props file not found in the log directory
      call :print_message "WARNING: Last collected file '%LAST_FILE_PROPS%' stored in props file was not found in '%LOG_PATH%'" "%ACTUATOR_LOG%"
      call :print_message "The file might have been deleted" "%ACTUATOR_LOG%"
      call :print_message "Possibly, many events were lost and not collected" "%ACTUATOR_LOG%"
      call :print_message "Collecting all events from log files closest to '%LAST_FILE_PROPS%' untill today, if any" "%ACTUATOR_LOG%"
    ) else (
        call :print_message "New %LOG_DESCRIPTION% log records found in '%LOG_PATH%'" "%ACTUATOR_LOG%"
    )
)
rem -- sets %DAYSAGO% equal to the %TIMESTAMP% of the file (without extension) stored in props file
for /F "delims=" %%i in ('@echo %LAST_FILE_PROPS%') do (
  <nul (set /P string=%%i)> %PROPS_FILE%.dir\string.len
  for %%j in ("%PROPS_FILE%.dir\string.len") do (
    set /A STRING_LENGTH=%%~zj-%TIMESTAMP_LENGTH%
  )
  call :find_timestamp "%%STRING_LENGTH%%" "%%i">nul 2>&1
  if NOT errorlevel 1 (
    call set DAYSAGO=%%TIMESTAMP%%
  )
)
if NOT exist "%LOG_PATH%%LAST_FILE_PROPS%" (
  call :prepare_copy_list "GEQ"
) else (
    call :prepare_copy_list "GTR"
)
if exist "%PROPS_FILE%.dir\%LOG_CATEGORY%_copy.bat" (
  call :execute_copy_command "POST_INITIAL"
) else (
    if exist "%PROPS_FILE%.dir\%LOG_CATEGORY%.chunk" (
      call ..\bin\bigchop.exe "200" "%PROPS_FILE%.dir\%LOG_CATEGORY%.chunk" "%TEMP_FILE%" "DOS"
      call :write_props "%LAST_FILE%" "%LAST_BYTE%" "%PROPS_FILE%.dir\%LOG_CATEGORY%.props"
    ) else (
        set FAILURE=post_initial_failure
    )
)
if defined FAILURE (
  goto %FAILURE%
) else (
    call :print_message "All records of %LOG_DESCRIPTION% log files were collected" "%ACTUATOR_LOG%"
    goto %NEXT_LABEL%
)

rem -- prints a message to the actuator log about no new records being found 
rem -- if no new recors in all files are available after reading props file
:post_initial_failure
call :print_message "No new records found in %LOG_DESCRIPTION% log files in '%LOG_PATH%'" "%ACTUATOR_LOG%"
type nul> "%TEMP_FILE%"
goto %NEXT_LABEL%

rem -- prints error message to the actuator log about inability to access props file
:props_failure
call :print_message "ERROR: Reading props file '%PROPS_FILE%' failure" "%ACTUATOR_LOG%"
set /A LOG_FAILURE+=1
goto %NEXT_LABEL%

rem ######################################################################################
rem props file creation routine
rem creates or update props file with data
rem takes three params: %1 - name of the last collected file, %2 - its size, %3 - name of actual or temporary props file to write the data to
rem ######################################################################################
:write_props
@echo %LOG_CATEGORY%^*%~1^*%~2 > %~3 || (call :print_message "ERROR: Temporary props file write failure" "%ACTUATOR_LOG%" & set FAILURE=failure)
goto :EOF

rem -- eof, collects temporary props files to %PROPS_FILE% and finishes script regarding the %LOG_FAILURE%
:eof
if exist %PROPS_FILE%.dir\*.props (
  copy /Y /B %PROPS_FILE%.dir\*.props /B %PROPS_FILE%> nul
)
if %LOG_FAILURE% GEQ 1 ( goto failure ) else ( goto success )

rem -- prints success message to actuator log and finishes the script with errorlevel 0 (SUCCESS)
:success
call :print_message "Processing finished [SUCCESS]" "%ACTUATOR_LOG%"
call :cleanup
exit /b 0

rem -- prints failure message to actuator log and finishes the script with errorlevel 1 (FAILURE)
:failure
call :print_message "Processing finished [FAILURE]" "%ACTUATOR_LOG%"
call :cleanup
exit /b 1

rem -- prints warning message about log files not being found
:logs_not_found
if NOT exist %PROPS_FILE% (
  call :print_message "This is the initial collect" "%ACTUATOR_LOG%"
  call :print_message "Collecting only events from %LOG_DESCRIPTION% log files for today and 2 previous days" "%ACTUATOR_LOG%"
) else (
    call :print_message "Reading props file '%PROPS_FILE%' to get the information about previous collect..." "%ACTUATOR_LOG%"
)
call :find_date "0"
call :write_props "%DAYSAGO%.AUD" "0" "%PROPS_FILE%.dir\%LOG_CATEGORY%.props"
call :print_message "WARNING: %LOG_DESCRIPTION% log files not found in '%LOG_PATH%'" "%ACTUATOR_LOG%"
type nul> "%TEMP_FILE%"
goto %NEXT_LABEL%

rem -- prints warning message about path containing illegal characters (<>|?*/)
:path_value_illegal
call :print_message "ERROR: Path for %LOG_DESCRIPTION% log files in ES properties contains illegal characters" "%ACTUATOR_LOG%"
set /A LOG_FAILURE+=1
goto %NEXT_LABEL%

rem -- prints warning message about path being empty
:path_value_empty
call :print_message "WARNING: Path for %LOG_DESCRIPTION% log files in ES properties is empty" "%ACTUATOR_LOG%"
type nul> "%TEMP_FILE%"
set /A LOG_FAILURE+=1
goto %NEXT_LABEL%

rem ######################################################################################
rem path expanding routine
rem dequotes path and appends '\' at the end of path if necessary
rem ######################################################################################
:path_value_expand
rem -- dequotes path
for /F "delims=" %%i in ('@echo %LOG_PATH%') do (set LOG_PATH=%%~fi)
rem -- checks if path ends with a backslash (ex: c:\some dir\) otherwise appends one at the end
(@echo %LOG_PATH% | findstr /R "\\\>"> nul && set LOG_PATH=%LOG_PATH%) || set LOG_PATH=%LOG_PATH%\
goto :EOF

rem ######################################################################################
rem path checking routine
rem checks path whether it contains illegal characters, empty or valid otherwise
rem ######################################################################################
:path_value_check
(set FAILURE=)
if NOT %LOG_PATH% == "" (
  rem -- checks if path contains illegal characters (<>?|*/)
  @echo %LOG_PATH% | findstr /R "[<>?*/|]"> nul && set FAILURE=path_value_illegal && goto :EOF
  call :path_value_expand
) else (
    rem -- path is empty
    set FAILURE=path_value_empty
    goto :EOF
)
exit /b 0
goto :EOF

rem ######################################################################################
rem printing routine
rem prints a message to actuator log. 
rem tekes two parameters: %1 - message itself, %2 - name of a log file
rem ######################################################################################
:print_message
if "%PRINTING%" == "enabled" (
  @echo [%date%][%time%] %~1>> %2
)
goto :EOF

rem ######################################################################################
rem cleanup routine
rem ######################################################################################
:cleanup
rem -- deletes temp directory
if exist %PROPS_FILE%.dir (
  rmdir /S /Q %PROPS_FILE%.dir
)
if exist %TEMP_LOG% (
  for %%i in (%TEMP_LOG%) do (
    if %%~zi EQU 0 (
      del /F/Q %TEMP_LOG%
    )
  )
)
goto :EOF
