#!/bin/ksh
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

# internal script usage settings
FIRST_PATH=`pwd`
ROOT_PATH=`dirname "$FIRST_PATH"`
LOG="$ROOT_PATH/log/$PROPSFILE.log"
# security audit log processing messages
MSG_LOG_INITIAL="New SAP Security Audit Log collect event started"
SA_MSG_LOG_PATH="ERROR: The SAP log path you've entered [$OPTION2] is invalid"
SA_MSG_ULOG_PERM="ERROR: Some old temporary log file [$ROOT_PATH/run/$TEMPFILE1] exists and cannot be deleted"
SA_MSG_ULOG_CR="ERROR: Temp log file creation failure in directory [$ROOT_PATH/run]"
SA_MSG_ULOG_DIRUNR="ERROR: Write attempt to [$ROOT_PATH/run] directory failed"
SA_MSG_ULOG_NODIR="ERROR: Can't write temp file to not existing directory [$ROOT_PATH/run]"
SA_MSG_PROPS="ERROR: Can't read file [$ROOT_PATH/run/$PROPSFILE]"
SA_MSG_PROPS_RW="ERROR: Writing file [$ROOT_PATH/run/$PROPSFILE] failure"
SA_MSG_PROPS_LUNR="WARNING: Log file stored in [$ROOT_PATH/run/$PROPSFILE] is unreadable"
SA_MSG_PROPS_CORR="Data which could be stored there is probably lost"
SA_MSG_LOG_ENTER="ERROR: Can't enter SAP log directory [$OPTION2]"
SA_MSG_LOG_UNR="ERROR: Log file is unreadable -"
SA_MSG_LOG_DIR="ERROR: Not file but directory is found -"
SA_MSG_LOG_DIR_EMPTY="WARNING: SAP log directory [$OPTION2] is empty"
SA_MSG_RSAU="Make sure the SAP Netweaver audit feature 'rsau/enable' to be on"
SA_MSG_LOG_PATH_EMPTY="WARNING: SAP log directory path setting is empty, please set valid path"
SA_MSG_TEMP_R="ERROR: Error occured while reading temporary file"
SA_MSG_INITIAL_COLLECT="This is initial collect, collecting events from 2 days ago till today only"
SA_MSG_NO2AGO="WARNING: There are no any log files for 2 last days"
SA_MSG_SUCCESS="Process finished [SUCCESS]"
SA_MSG_FAILURE="Process finished [FAILURE]"
SA_SEARCHING="Searching new log events in"
SA_NONEW="No new events to be collected"

# logging functions
startlog()
{
if ([ "`uname`" = "AIX" ] || [ "`uname`" = "HP-UX" ] || [ "`uname`" = "SunOS" ]); then
    echo "`date +\[%d.%m.%Y\]\[%T\]` $1" > "$LOG"
fi
if [ "`uname`" = "Linux" ]; then
    echo -en "`date +\[%d.%m.%Y\]\[%T\]` $1\n" > "$LOG"
fi
}
writelog()
{
if ([ "`uname`" = "AIX" ] || [ "`uname`" = "HP-UX" ] || [ "`uname`" = "SunOS" ]); then
    echo "`date +\[%d.%m.%Y\]\[%T\]` $1" >> "$LOG"
fi
if [ "`uname`" = "Linux" ]; then
    echo -en "`date +\[%d.%m.%Y\]\[%T\]` $1\n" >> "$LOG"
fi
}

# function returns the date ago
gobackintime()
{
CURRENT_YEAR=`date +%Y`
CURRENT_MONTH=`date +%m`
CURRENT_DAY=`date +%d`

# checking for leap year
LEAP=0

if [ `expr $CURRENT_YEAR % 400` -eq 0 ] ; then
 LEAP=1
else
 if [ `expr $CURRENT_YEAR % 100` -eq 0 ] ; then 
   LEAP=0
 else
  if [ `expr $CURRENT_YEAR % 4` -eq 0 ] ; then
   LEAP=1
  fi  
 fi 
fi

NEW_DAY=`expr $CURRENT_DAY - 2`
NEW_MONTH=$CURRENT_MONTH
NEW_YEAR=$CURRENT_YEAR

if [ $NEW_DAY -lt 1 ] ; then
case $CURRENT_MONTH in
 1)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=12
   NEW_YEAR=`expr $CURRENT_YEAR - 1`
   ;;
 2)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 3)
   if [ $LEAP -eq 1 ] ; then
    NEW_DAY=`expr $NEW_DAY + 29`
    NEW_MONTH=`expr $CURRENT_MONTH - 1`
   else
    NEW_DAY=`expr $NEW_DAY + 28`
    NEW_MONTH=`expr $CURRENT_MONTH - 1`
   fi
   ;;
 4)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 5)
   NEW_DAY=`expr $NEW_DAY + 30`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 6)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 7)
   NEW_DAY=`expr $NEW_DAY + 30`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 8)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 9)
   NEW_DAY=`expr $NEW_DAY + 30`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 10)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 11)
   NEW_DAY=`expr $NEW_DAY + 30`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
 12)
   NEW_DAY=`expr $NEW_DAY + 31`
   NEW_MONTH=`expr $CURRENT_MONTH - 1`
   ;;
   *) echo "Error checking month number, process finished [FAILURE]" > $LOG && exit 1
   ;;
esac
fi

# adding "0" character if number contains only one digit
NEW_MONTH=`echo $NEW_MONTH | sed "s/^[1-9]$/0\$NEW_MONTH/"`
NEW_DAY=`echo $NEW_DAY | sed "s/^[1-9]$/0\$NEW_DAY/"`
echo "$NEW_YEAR$NEW_MONTH$NEW_DAY"
}

# == SA ==================== security audit log ==================== SA ==
# PATHS
SA_2AGO=`gobackintime`                               # two days ago date
SA_TODAY=`date +%Y%m%d`                              # today's date
SA_LOG_PATH="$OPTION2"                               # log file path
SA_ULOG="$TEMPFILE1"                                 # temporary file
SA_PROPS_FILE="$ROOT_PATH/run/$PROPSFILE"            # props file
SA_TAIL="tail"
SA_AWK="awk"
SA_REC_LENGTH=200
SA_NEW_EVENTS="0"
if [ "`uname`" = "SunOS" ]; then
    SA_TAIL="/usr/xpg4/bin/tail"
    SA_AWK="/usr/xpg4/bin/awk"
fi
if [ "`dirname "$SA_ULOG"`" = "." ]; then
    SA_ULOG="$ROOT_PATH/run/$TEMPFILE1"
fi

# adding initial log entry
startlog "$MSG_LOG_INITIAL"
writelog "$SA_SEARCHING [$SA_LOG_PATH]"

# VALIDATING TEMPORARY LOG FILE
# if there exists old file and can be deleted
if [ ! -d "`dirname "$SA_ULOG"`" ]; then writelog "$SA_MSG_ULOG_NODIR"; writelog "$SA_MSG_FAILURE"; exit 1; fi
if [ ! -w "`dirname "$SA_ULOG"`" ]; then writelog "$SA_MSG_ULOG_DIRUNR"; writelog "$SA_MSG_FAILURE"; exit 1; fi
if [ -f "$SA_ULOG" ]; then
    if [ -w "$SA_ULOG" ]; then rm -rf $SA_ULOG; fi
    # if is file and still exists after attempt to delete
    if [ -f "$SA_ULOG" ]; then writelog "$SA_MSG_ULOG_PERM"; writelog "$SA_MSG_FAILURE"; exit 1; fi
fi
touch "$SA_ULOG" 2> /dev/null
if [ $? -ne 0 ]; then
    writelog "$SA_MSG_ULOG_CR"
    writelog "$SA_MSG_FAILURE";
    exit 1
fi

# VALIDATING LOG DIRECTORY
# in case of empty path
if [ "$SA_LOG_PATH" = "" ]; then writelog "$SA_MSG_LOG_PATH_EMPTY"; writelog "$SA_MSG_FAILURE"; exit 1; fi
# if directory exists
if [ ! -d "$SA_LOG_PATH" ]; then writelog "$SA_MSG_LOG_PATH"; writelog "$SA_MSG_FAILURE"; exit 1; fi
# checking for stars to be absent
TMP_VALIDITY=`echo "$SA_LOG_PATH" | grep "*"`
if [ "$TMP_VALIDITY" != "" ]; then writelog "$SA_MSG_LOG_PATH"; writelog "$SA_MSG_FAILURE"; exit 1; fi
# checking the folder to be available to enter
if [ ! -x "$SA_LOG_PATH" ]; then writelog "$SA_MSG_LOG_ENTER"; writelog "$SA_MSG_FAILURE"; exit 1; fi
# checking if log directory is empty
TMP_LOGDIR=`ls -1r "$SA_LOG_PATH"`
if [ "$TMP_LOGDIR" = "" ]; then
    writelog "$SA_MSG_LOG_DIR_EMPTY"
    writelog "$SA_MSG_RSAU"
fi

# in case when props file does not exist - initial collect
if [ ! -f "$SA_PROPS_FILE" ]; then
    if [ -w "`dirname "$SA_PROPS_FILE"`" ]; then
	touch "$SA_PROPS_FILE" 2> /dev/null
    else
            writelog "$SA_MSG_PROPS_RW"; writelog "$SA_MSG_FAILURE"; exit 1;
    fi
    if [ $? -ne 0 ]; then
	    writelog "$SA_MSG_PROPS_RW"; writelog "$SA_MSG_FAILURE"; exit 1;
    fi
    if [ ! -w "$SA_PROPS_FILE" ]; then writelog "$SA_MSG_PROPS_RW"; writelog "$SA_MSG_FAILURE"; exit 1; fi
    SA_MIDFILE=$SA_2AGO
fi
# in case when AUDIT record doesn't exist in the file - initial collect
TMP_AUDIT_SET=`cat "$SA_PROPS_FILE" | grep "AUDIT"`
if [ "$TMP_AUDIT_SET" = "" ]; then
    if [ ! -w "$SA_PROPS_FILE" ]; then writelog "$SA_MSG_PROPS_RW"; writelog "$SA_MSG_FAILURE"; exit 1; fi
    writelog "$SA_MSG_INITIAL_COLLECT";
    SA_MIDFILE=$SA_2AGO
fi

# if it is not initial collect
if [ "$TMP_AUDIT_SET" != "" ]; then
    # reading LAST_FILE and LAST_BYTE
    if [ ! -r "$SA_PROPS_FILE" ]; then writelog "$SA_MSG_PROPS"; writelog "$SA_MSG_FAILURE"; exit 1; fi
    TMP_PROPS_AUDIT=`cat "$SA_PROPS_FILE" | grep AUDIT`
    SA_LAST_FILE=`echo $TMP_PROPS_AUDIT | cut -d " " -f 2`
    SA_LAST_BYTE=`echo $TMP_PROPS_AUDIT | cut -d " " -f 3`
    # checking if additional reading needed
    if [ ! -r "$SA_LOG_PATH/$SA_LAST_FILE" ]; then
	if [ "$SA_LAST_BYTE" != "0" ]; then
	    writelog "$SA_MSG_PROPS_LUNR"
	    writelog "$SA_MSG_PROPS_CORR"
	fi
    fi
    if [ -r "$SA_LOG_PATH/$SA_LAST_FILE" ]; then
	SA_CURR_SIZE=`ls -l "$SA_LOG_PATH/$SA_LAST_FILE" | $SA_AWK '{print $5}'`
	if [ "$SA_CURR_SIZE" -gt "$SA_LAST_BYTE" ]; then
	    writelog "Processing file $SA_LAST_FILE"
	    SA_NEW_EVENTS="1"
	    $SA_TAIL -c `expr "$SA_CURR_SIZE" "-" "$SA_LAST_BYTE"` "$SA_LOG_PATH/$SA_LAST_FILE" >> "$SA_ULOG"
	    # case of error
	    if [ $? -ne 0 ]; then
		writelog "$SA_MSG_ULOG_CR"
		writelog "$SA_MSG_FAILURE"
		exit 1
	    fi
	fi
    fi
    SA_MIDFILE=`echo $SA_LAST_FILE | $SA_AWK '{ gsub("[^0-9]","",$0); print $0 }'`
fi

# cycling all list to find new log files
SA_NEW_LOGS=""
SA_NEW_LAST=""
cd "$SA_LOG_PATH"
for SA_LOG_FULL in `ls -1r *[0-9][0-9][0-9][0-9][0-1][0-9][0-3][0-9]*`
do
    if [ -d "$SA_LOG_FULL" ]; then writelog "$SA_MSG_LOG_DIR $SA_LOG_FULL"; writelog "$SA_MSG_FAILURE"; exit 1; fi
    if [ ! -r "$SA_LOG_FULL" ]; then writelog "$SA_MSG_LOG_UNR $SA_LOG_FULL"; writelog "$SA_MSG_FAILURE"; exit 1; fi
    SA_LOG_SIMPLE=`echo $SA_LOG_FULL | $SA_AWK '{ gsub("[^0-9]","",$0); print $0 }'`
    if ([ "$SA_LOG_SIMPLE" -ge "$SA_MIDFILE" ] && [ "$SA_LOG_SIMPLE" -le "$SA_TODAY" ] && [ "$SA_LOG_FULL" != "$SA_LAST_FILE" ]); then
	writelog "Processing file $SA_LOG_FULL"
	SA_NEW_EVENTS="1"
	SA_NEW_LOGS="$SA_LOG_FULL $SA_NEW_LOGS"
	SA_NEW_LAST="$SA_NEW_LAST $SA_LOG_FULL";
    fi
done

# uniting all needed logs into one file
if [ "$SA_NEW_LOGS" != "" ]; then
    cat $SA_NEW_LOGS >> $SA_ULOG
    # case of error
    if [ $? -ne 0 ]; then
	writelog "$SA_MSG_TEMP_R"
	writelog "$SA_MSG_FAILURE"
	exit 1
    fi
fi

# VALIDATING PROPS FILE
# if it is available for writing
if [ ! -w "$SA_PROPS_FILE" ]; then writelog "$SA_MSG_PROPS_RW"; writelog "$SA_MSG_FAILURE"; exit 1; fi

# WRITING PROPS FILE
if [ "$SA_NEW_LAST" != "" ]; then SA_NEW_LAST_FILE=`echo $SA_NEW_LAST | cut -d " " -f 1`;
else SA_NEW_LAST_FILE=$SA_LAST_FILE; fi

# forming
TMP_OTHER_RECS=""
if [ -r "$SA_NEW_LAST_FILE" ]; then
    SA_NEW_LAST_SIZE=`ls -l $SA_NEW_LAST_FILE | $SA_AWK '{print $5}'`
    SA_PROPS_OUT="AUDIT $SA_NEW_LAST_FILE $SA_NEW_LAST_SIZE"
    TMP_OTHER_RECS=`cat $SA_PROPS_FILE | grep -v AUDIT`
else
    TMP_OTHER_RECS=`cat $SA_PROPS_FILE | grep -v AUDIT`
    SA_PROPS_OUT="AUDIT `date +%Y%m%d`.AUD 0"
fi

# writing
if [ "$TMP_OTHER_RECS" != "" ]; then
    echo $TMP_OTHER_RECS > "$SA_PROPS_FILE";
    if ([ "`uname`" = "AIX" ] || [ "`uname`" = "HP-UX" ] || [ "`uname`" = "SunOS" ]); then echo $SA_PROPS_OUT >> "$SA_PROPS_FILE"; fi
    if [ "`uname`" = "Linux" ]; then echo -n $SA_PROPS_OUT >> "$SA_PROPS_FILE"; fi
else
    if ([ "`uname`" = "AIX" ] || [ "`uname`" = "HP-UX" ] || [ "`uname`" = "SunOS" ]); then echo $SA_PROPS_OUT > "$SA_PROPS_FILE"; fi
    if [ "`uname`" = "Linux" ]; then echo -n $SA_PROPS_OUT > "$SA_PROPS_FILE"; fi
fi

cd $FIRST_PATH

# CHOPPING CHUNK
# obtaining chunk size
SA_ULOG_SIZE=`ls -l "$SA_ULOG" | $SA_AWK '{print $5}'`
SA_2RECS_LENGTH=`expr "$SA_REC_LENGTH" "*" "2"`
# if it contains at least two records, chop it then
if [ "$SA_ULOG_SIZE" -ge "$SA_2RECS_LENGTH" ]; then
    mv -f "$SA_ULOG" "$ROOT_PATH/run/__saulog.tmp"
    fold -w "$SA_REC_LENGTH" "$ROOT_PATH/run/__saulog.tmp" > "$SA_ULOG"
    rm -f "$ROOT_PATH/run/__saulog.tmp"
fi

# done
if [ "$SA_NEW_EVENTS" = "0" ]; then writelog "$SA_NONEW"; fi
if ([ "$SA_MIDFILE" = "$SA_2AGO" ] && [ "$SA_NEW_LAST" = "" ]); then
    writelog "$SA_MSG_NO2AGO"
    writelog "$SA_MSG_SUCCESS"
else
    writelog "$SA_MSG_SUCCESS"
fi

exit 0
